@@ -16,33 +16,28 @@ gi.require_version("GstAudio", "1.0")
 gi.require_version("GstPbutils", "1.0")
 gi.require_version("TotemPlParser", "1.0")
 from gi.repository import Gtk, Gio, GLib, Gdk, Gst, GstPbutils
+from gi.repository.Gio import FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE
 Gst.init(None)
 GstPbutils.pb_utils_init()
 
 from threading import current_thread
 from pickle import dump
 from signal import signal, SIGINT, SIGTERM
+from urllib.parse import urlparse
 
-
-try:
-    from lollypop.lastfm import LastFM, LibreFM
-except Exception as e:
-    print(e)
-    print("$ sudo pip3 install pylast")
-    LastFM = None
-
-from lollypop.utils import init_proxy_from_gnome, install_youtube_dl
+from lollypop.utils import init_proxy_from_gnome, emit_signal
 from lollypop.application_actions import ApplicationActions
-from lollypop.utils import is_audio, is_pls, emit_signal
-from lollypop.define import Type, LOLLYPOP_DATA_PATH, ScanType, StorageType
+from lollypop.utils_file import is_audio, is_pls, install_youtube_dl
+from lollypop.define import LOLLYPOP_DATA_PATH, ScanType, StorageType
 from lollypop.database import Database
 from lollypop.player import Player
 from lollypop.inhibitor import Inhibitor
 from lollypop.art import Art
 from lollypop.logger import Logger
-from lollypop.search_spotify import SpotifySearch
+from lollypop.ws_director import DirectorWebService
 from lollypop.sqlcursor import SqlCursor
 from lollypop.settings import Settings
+from lollypop.database_cache import CacheDatabase
 from lollypop.database_albums import AlbumsDatabase
 from lollypop.database_artists import ArtistsDatabase
 from lollypop.database_genres import GenresDatabase
@@ -51,11 +46,9 @@ from lollypop.notification import NotificationManager
 from lollypop.playlists import Playlists
 from lollypop.objects_track import Track
 from lollypop.objects_album import Album
-from lollypop.objects_radio import Radio
-from lollypop.radios import Radios
 from lollypop.helper_task import TaskHelper
 from lollypop.helper_art import ArtHelper
-from lollypop.collectionscanner import CollectionScanner
+from lollypop.collection_scanner import CollectionScanner
 
 
 class Application(Gtk.Application, ApplicationActions):
@@ -100,14 +93,10 @@ class Application(Gtk.Application, ApplicationActions):
                     GLib.setenv("SSL_CERT_FILE", path, True)
                     break
         self.cursors = {}
-        self.scrobblers = []
         self.debug = False
         self.shown_sidebar_tooltip = False
         self.__window = None
         self.__fs_window = None
-        self.__scanner_timeout_id = None
-        self.__spotify_timeout_id = None
-        self.__scanner_uris = []
         settings = Gio.Settings.new("org.gnome.desktop.interface")
         self.animations = settings.get_value("enable-animations").get_boolean()
         GLib.set_application_name("Lollypop")
@@ -178,12 +167,12 @@ class Application(Gtk.Application, ApplicationActions):
         styleContext.add_provider_for_screen(screen, cssProvider,
                                              Gtk.STYLE_PROVIDER_PRIORITY_USER)
         self.db = Database()
+        self.cache = CacheDatabase()
         self.playlists = Playlists()
-        self.albums = AlbumsDatabase()
-        self.artists = ArtistsDatabase()
-        self.genres = GenresDatabase()
-        self.tracks = TracksDatabase()
-        self.radios = Radios()
+        self.albums = AlbumsDatabase(self.db)
+        self.artists = ArtistsDatabase(self.db)
+        self.genres = GenresDatabase(self.db)
+        self.tracks = TracksDatabase(self.db)
         self.player = Player()
         self.inhibitor = Inhibitor()
         self.scanner = CollectionScanner()
@@ -192,8 +181,8 @@ class Application(Gtk.Application, ApplicationActions):
         self.art_helper = ArtHelper()
         self.art = Art()
         self.art.update_art_size()
-        self.art.clean_old_artwork()
-        self.spotify = SpotifySearch()
+        self.ws_director = DirectorWebService()
+        self.ws_director.start()
         if not self.settings.get_value("disable-mpris"):
             from lollypop.mpris import MPRIS
             MPRIS(self)
@@ -210,28 +199,6 @@ class Application(Gtk.Application, ApplicationActions):
                 not monitor.get_network_metered() and\
                 self.settings.get_value("recent-youtube-dl"):
             self.task_helper.run(install_youtube_dl)
-        self.start_spotify()
-
-    def start_spotify(self):
-        """
-            Start spotify timeout and start a new populate
-        """
-        if Type.SUGGESTIONS not in\
-                self.settings.get_value("shown-album-lists"):
-            return
-        if self.__spotify_timeout_id is None:
-            self.spotify.populate_db()
-            self.__spotify_timeout_id = GLib.timeout_add_seconds(
-                3600, self.spotify.populate_db)
-
-    def stop_spotify(self):
-        """
-            Stop spotify timeout and stop current populate
-        """
-        if self.__spotify_timeout_id is not None:
-            self.spotify.stop()
-            GLib.source_remove(self.__spotify_timeout_id)
-            self.__spotify_timeout_id = None
 
     def do_startup(self):
         """
@@ -252,9 +219,9 @@ class Application(Gtk.Application, ApplicationActions):
             Quit Lollypop
             @param vacuum as bool
         """
+        self.__window.container.stop()
         self.__window.hide()
-        if self.spotify.is_running:
-            self.spotify.stop()
+        if not self.ws_director.stop():
             GLib.timeout_add(100, self.quit, vacuum)
             return
         if self.settings.get_value("save-state"):
@@ -262,9 +229,7 @@ class Application(Gtk.Application, ApplicationActions):
         # Then vacuum db
         if vacuum:
             self.__vacuum()
-            self.art.clean_web()
-        for scrobbler in self.scrobblers:
-            scrobbler.save()
+            self.art.clean_artwork()
         Gio.Application.quit(self)
         if GLib.environ_getenv(GLib.get_environ(), "DEBUG_LEAK") is not None:
             import gc
@@ -273,20 +238,6 @@ class Application(Gtk.Application, ApplicationActions):
                 s = str(x)
                 print(type(x), "\n  ", s)
 
-    def load_listenbrainz(self):
-        """
-            Load listenbrainz support if needed
-        """
-        if self.settings.get_value("listenbrainz-user-token").get_string():
-            from lollypop.listenbrainz import ListenBrainz
-            for scrobbler in self.scrobblers:
-                if isinstance(scrobbler, ListenBrainz):
-                    return
-            listenbrainz = ListenBrainz()
-            self.scrobblers.append(listenbrainz)
-            self.settings.bind("listenbrainz-user-token", listenbrainz,
-                               "user_token", 0)
-
     def fullscreen(self):
         """
             Go fullscreen
@@ -296,12 +247,14 @@ class Application(Gtk.Application, ApplicationActions):
             self.__window.show()
 
         if self.__fs_window is None:
-            self.__window.hide()
             from lollypop.fullscreen import FullScreen
             self.__fs_window = FullScreen()
             self.__fs_window.delayed_init()
             self.__fs_window.show()
             self.__fs_window.connect("destroy", on_destroy)
+            self.__window.hide()
+        else:
+            self.__fs_window.destroy()
 
     @property
     def proxy_host(self):
@@ -342,20 +295,6 @@ class Application(Gtk.Application, ApplicationActions):
         """
         return self.__fs_window is not None
 
-    @property
-    def lastfm(self):
-        """
-            Get lastfm provider from scrobbler
-            @return LastFM/None
-        """
-        if LastFM is None:
-            return None
-        from pylast import LastFMNetwork
-        for scrobbler in self.scrobblers:
-            if isinstance(scrobbler, LastFMNetwork):
-                return scrobbler
-        return None
-
     @property
     def main_window(self):
         """
@@ -403,41 +342,32 @@ class Application(Gtk.Application, ApplicationActions):
 #######################
     def __save_state(self):
         """
-            Save window position and view
+            Save player state
         """
-        if not self.settings.get_value("save-state"):
-            return
-
-        if self.player.current_track.id is None or\
-                self.player.current_track.storage_type & StorageType.EPHEMERAL:
-            track_id = None
-        else:
-            track_id = self.player.current_track.id
-            # Save albums context
-            try:
-                with open(LOLLYPOP_DATA_PATH + "/Albums.bin", "wb") as f:
-                    dump(self.player.albums, f)
-            except Exception as e:
-                Logger.error("Application::__save_state(): %s" % e)
-        dump(track_id, open(LOLLYPOP_DATA_PATH + "/track_id.bin", "wb"))
-        dump([self.player.is_playing, self.player.is_party],
-             open(LOLLYPOP_DATA_PATH + "/player.bin", "wb"))
-        dump(self.player.queue,
-             open(LOLLYPOP_DATA_PATH + "/queue.bin", "wb"))
-        # Save current playlist
-        if isinstance(self.player.current_track, Radio):
-            playlist_ids = [Type.RADIOS]
-        else:
-            playlist_ids = []
-        dump(playlist_ids,
-             open(LOLLYPOP_DATA_PATH + "/playlist_ids.bin", "wb"))
-        if self.player.current_track.id is not None:
-            position = self.player.position
-        else:
-            position = 0
-        dump(position, open(LOLLYPOP_DATA_PATH + "/position.bin", "wb"))
+        if self.settings.get_value("save-state"):
+            if self.player.current_track.id is None or\
+                    self.player.current_track.storage_type &\
+                    StorageType.EPHEMERAL:
+                track_id = None
+            else:
+                track_id = self.player.current_track.id
+                # Save albums context
+                try:
+                    with open(LOLLYPOP_DATA_PATH + "/Albums.bin", "wb") as f:
+                        dump(self.player.albums, f)
+                except Exception as e:
+                    Logger.error("Application::__save_state(): %s" % e)
+            dump(track_id, open(LOLLYPOP_DATA_PATH + "/track_id.bin", "wb"))
+            dump([self.player.is_playing, self.player.is_party],
+                 open(LOLLYPOP_DATA_PATH + "/player.bin", "wb"))
+            dump(self.player.queue,
+                 open(LOLLYPOP_DATA_PATH + "/queue.bin", "wb"))
+            if self.player.current_track.id is not None:
+                position = self.player.position
+            else:
+                position = 0
+            dump(position, open(LOLLYPOP_DATA_PATH + "/position.bin", "wb"))
         self.player.stop_all()
-        self.__window.container.stop_all()
 
     def __vacuum(self):
         """
@@ -454,10 +384,9 @@ class Application(Gtk.Application, ApplicationActions):
             self.albums.clean(False)
             self.artists.clean(False)
             self.genres.clean(False)
-            SqlCursor.commit(self.db)
             SqlCursor.remove(self.db)
+            self.cache.clean(True)
 
-            from lollypop.radios import Radios
             with SqlCursor(self.db) as sql:
                 sql.isolation_level = None
                 sql.execute("VACUUM")
@@ -466,13 +395,25 @@ class Application(Gtk.Application, ApplicationActions):
                 sql.isolation_level = None
                 sql.execute("VACUUM")
                 sql.isolation_level = ""
-            with SqlCursor(Radios()) as sql:
-                sql.isolation_level = None
-                sql.execute("VACUUM")
-                sql.isolation_level = ""
         except Exception as e:
             Logger.error("Application::__vacuum(): %s" % e)
 
+    def __parse_uris(self, playlist_uris, audio_uris):
+        """
+            Parse playlist uris
+            @param playlist_uris as [str]
+            @param audio_uris as [str]
+        """
+        from gi.repository import TotemPlParser
+        playlist_uri = playlist_uris.pop(0)
+        parser = TotemPlParser.Parser.new()
+        parser.connect("entry-parsed",
+                       self.__on_entry_parsed,
+                       audio_uris)
+        parser.parse_async(playlist_uri, True, None,
+                           self.__on_parse_finished,
+                           playlist_uris, audio_uris)
+
     def __on_handle_local_options(self, app, options):
         """
             Handle local options
@@ -495,11 +436,6 @@ class Application(Gtk.Application, ApplicationActions):
             options = app_cmd_line.get_options_dict()
             if options.contains("debug"):
                 self.debug = True
-            # We are forced to enable scrobblers here if we want full debug
-            if not self.scrobblers:
-                if LastFM is not None:
-                    self.scrobblers = [LastFM(), LibreFM()]
-                self.load_listenbrainz()
             if options.contains("set-rating"):
                 value = options.lookup_value("set-rating").get_string()
                 try:
@@ -517,16 +453,18 @@ class Application(Gtk.Application, ApplicationActions):
                 try:
                     value = options.lookup_value("play-ids").get_string()
                     ids = value.split(";")
+                    albums = []
                     for id in ids:
                         if id[0:2] == "a:":
                             album = Album(int(id[2:]))
                             self.player.add_album(album)
+                            albums.append(album)
                         else:
                             track = Track(int(id[2:]))
-                            track.album.set_tracks([track])
                             self.player.add_album(track.album)
-                    if self.player.albums and self.player.albums[0].tracks:
-                        self.player.load(self.player.albums[0].tracks[0])
+                            albums.append(track.album)
+                    if albums and albums[0].tracks:
+                        self.player.load(albums[0].tracks[0])
                 except Exception as e:
                     Logger.error("Application::__on_command_line(): %s", e)
                     pass
@@ -537,8 +475,8 @@ class Application(Gtk.Application, ApplicationActions):
             elif options.contains("emulate-phone"):
                 self.__window.toolbar.end.devices_popover.add_fake_phone()
             elif len(args) > 1:
-                uris = []
-                pls = []
+                audio_uris = []
+                playlist_uris = []
                 for uri in args[1:]:
                     try:
                         uri = GLib.filename_to_uri(uri)
@@ -549,25 +487,18 @@ class Application(Gtk.Application, ApplicationActions):
                         uri = GLib.filename_to_uri(
                             "%s/%s" % (GLib.get_current_dir(), uri))
                         f = Gio.File.new_for_uri(uri)
-                    if is_audio(f):
-                        uris.append(uri)
-                    elif is_pls(f):
-                        pls.append(uri)
-                    else:
-                        info = f.query_info(Gio.FILE_ATTRIBUTE_STANDARD_TYPE,
-                                            Gio.FileQueryInfoFlags.NONE,
-                                            None)
-                        if info.get_file_type() == Gio.FileType.DIRECTORY:
-                            uris.append(uri)
-                if pls:
-                    from gi.repository import TotemPlParser
-                    parser = TotemPlParser.Parser.new()
-                    parser.connect("entry-parsed",
-                                   self.__on_entry_parsed, uris)
-                    parser.parse_async(uri, True, None,
-                                       self.__on_parse_finished, uris)
+                    info = f.query_info(FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+                                        Gio.FileQueryInfoFlags.NONE)
+                    if is_audio(info):
+                        audio_uris.append(uri)
+                    elif is_pls(info):
+                        playlist_uris.append(uri)
+                    elif info.get_file_type() == Gio.FileType.DIRECTORY:
+                        audio_uris.append(uri)
+                if playlist_uris:
+                    self.__parse_uris(playlist_uris, audio_uris)
                 else:
-                    self.__on_parse_finished(None, None, uris)
+                    self.__on_parse_finished(None, None, [], audio_uris)
             elif self.__window is not None:
                 if not self.__window.is_visible():
                     self.__window.present()
@@ -578,34 +509,34 @@ class Application(Gtk.Application, ApplicationActions):
             Logger.error("Application::__on_command_line(): %s", e)
         return 0
 
-    def __on_parse_finished(self, parser, result, uris):
+    def __on_parse_finished(self, source, result, playlist_uris, audio_uris):
         """
             Play stream
-            @param parser as TotemPlParser.Parser
+            @param source as None
             @param result as Gio.AsyncResult
-            @param uris as [str]
+            @param uris as ([str], [str])
         """
-        def scanner_update():
-            self.__scanner_timeout_id = None
-            self.player.play_uris(self.__scanner_uris)
-            self.scanner.update(ScanType.EXTERNAL, self.__scanner_uris)
-            self.__scanner_uris = []
+        if playlist_uris:
+            self.__parse_uris(playlist_uris, audio_uris)
+        else:
+            to_scan_uris = []
+            for uri in audio_uris:
+                parsed = urlparse(uri)
+                if parsed.scheme in ["http", "https"]:
+                    pass
+                else:
+                    to_scan_uris.append(uri)
+            self.scanner.update(ScanType.EXTERNAL, to_scan_uris)
 
-        if self.__scanner_timeout_id is not None:
-            GLib.source_remove(self.__scanner_timeout_id)
-        self.__scanner_uris += uris
-        self.__scanner_timeout_id = GLib.timeout_add(500,
-                                                     scanner_update)
-
-    def __on_entry_parsed(self, parser, uri, metadata, uris):
+    def __on_entry_parsed(self, parser, uri, metadata, audio_uris):
         """
             Add playlist entry to external files
             @param parser as TotemPlParser.Parser
             @param uri as str
             @param metadata as GLib.HastTable
-            @param uris as str
+            @param audio_uris as str
         """
-        uris.append(uri)
+        audio_uris.append(uri)
 
     def __hide_on_delete(self, widget, event):
         """
