@@ -13,16 +13,15 @@
 from gi.repository import GLib, GdkPixbuf, Gio, Gst
 
 from random import choice
-from hashlib import md5
 from gettext import gettext as _
 from time import time
 
 from lollypop.tagreader import Discoverer
 from lollypop.define import App, ArtSize, ArtBehaviour, StorageType
-from lollypop.define import CACHE_PATH, TMP_PATH, STORE_PATH
-from lollypop.objects_album import Album
+from lollypop.define import CACHE_PATH, ALBUMS_WEB_PATH, ALBUMS_PATH
 from lollypop.logger import Logger
-from lollypop.utils import is_readonly, emit_signal
+from lollypop.utils_file import is_readonly
+from lollypop.utils import emit_signal
 from lollypop.helper_task import TaskHelper
 
 
@@ -52,11 +51,9 @@ class AlbumArt:
             @param height as int
             @return cover path as string or None if no cover
         """
-        filename = ""
         try:
-            filename = self.get_album_cache_name(album)
             cache_path_jpg = "%s/%s_%s_%s.jpg" % (CACHE_PATH,
-                                                  filename,
+                                                  album.lp_album_id,
                                                   width,
                                                   height)
             f = Gio.File.new_for_path(cache_path_jpg)
@@ -82,20 +79,20 @@ class AlbumArt:
         if album.id is None:
             return None
         try:
-            filename = self.get_album_cache_name(album) + ".jpg"
             self.__update_album_uri(album)
-            if album.storage_type & StorageType.EPHEMERAL:
-                store_path = TMP_PATH + "/" + filename
+            if not album.storage_type & StorageType.COLLECTION:
+                store_path = "%s/%s.jpg" % (ALBUMS_WEB_PATH, album.lp_album_id)
+                uris = [GLib.filename_to_uri(store_path)]
             else:
-                store_path = STORE_PATH + "/" + filename
-            uris = [
-                # Used when album.uri is readonly or for Web
-                GLib.filename_to_uri(store_path),
-                # Default favorite artwork
-                album.uri + "/" + self.__favorite,
-                # Used when having muliple albums in same folder
-                album.uri + "/" + filename
-            ]
+                store_path = "%s/%s.jpg" % (ALBUMS_PATH, album.lp_album_id)
+                uris = [
+                    # Default favorite artwork
+                    "%s/%s" % (album.uri, self.__favorite),
+                    # Used when album.uri is readonly or for Web
+                    GLib.filename_to_uri(store_path),
+                    # Used when having muliple albums in same folder
+                    "%s/%s.jpg" % (album.uri, album.lp_album_id)
+                ]
             for uri in uris:
                 f = Gio.File.new_for_uri(uri)
                 if f.query_exists():
@@ -113,6 +110,9 @@ class AlbumArt:
         # Folders with many albums, get_album_artwork_uri()
         if App().albums.get_uri_count(album.uri) > 1:
             return None
+        if not album.storage_type & (StorageType.COLLECTION |
+                                     StorageType.EXTERNAL):
+            return None
         f = Gio.File.new_for_uri(album.uri)
         infos = f.enumerate_children("standard::name",
                                      Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS,
@@ -132,8 +132,11 @@ class AlbumArt:
             @param album as Album
             @return [paths]
         """
-        uris = []
+        if not album.storage_type & (StorageType.COLLECTION |
+                                     StorageType.EXTERNAL):
+            return []
         try:
+            uris = []
             f = Gio.File.new_for_uri(album.uri)
             infos = f.enumerate_children(
                 "standard::name",
@@ -164,11 +167,11 @@ class AlbumArt:
             @return cairo surface
             @thread safe
         """
+        uri = None
         if album.id is None:
             return None
         width *= scale_factor
         height *= scale_factor
-        filename = self.get_album_cache_name(album)
         # Blur when reading from tags can be slow, so prefer cached version
         # Blur allows us to ignore width/height until we want CROP/CACHE
         optimized_blur = behaviour & (ArtBehaviour.BLUR |
@@ -182,7 +185,8 @@ class AlbumArt:
         else:
             w = width
             h = height
-        cache_path_jpg = "%s/%s_%s_%s.jpg" % (CACHE_PATH, filename, w, h)
+        cache_path_jpg = "%s/%s_%s_%s.jpg" % (CACHE_PATH, album.lp_album_id,
+                                              w, h)
         pixbuf = None
         try:
             # Look in cache
@@ -207,19 +211,19 @@ class AlbumArt:
                             stream, None)
                         stream.close()
                 # Use tags artwork
-                if pixbuf is None and album.tracks and album.uri != "":
+                if pixbuf is None and album.tracks and\
+                        album.storage_type & (StorageType.COLLECTION |
+                                              StorageType.EXTERNAL):
                     try:
-                        if behaviour & (ArtBehaviour.BLUR |
-                                        ArtBehaviour.BLUR_HARD):
-                            track = album.tracks[0]
-                        else:
-                            track = choice(album.tracks)
+                        track = choice(album.tracks)
                         pixbuf = self.pixbuf_from_tags(track.uri)
                     except Exception as e:
                         Logger.error("AlbumArt::get_album_artwork(): %s", e)
 
                 # Use folder artwork
-                if pixbuf is None and album.uri != "":
+                if pixbuf is None and\
+                        album.storage_type & (StorageType.COLLECTION |
+                                              StorageType.EXTERNAL):
                     uri = self.get_first_album_artwork(album)
                     # Look in album folder
                     if uri is not None:
@@ -237,42 +241,43 @@ class AlbumArt:
                                              width, height, behaviour)
                 return pixbuf
         except Exception as e:
-            Logger.error("AlbumArt::get_album_artwork(): %s" % e)
+            Logger.error("AlbumArt::get_album_artwork(): %s -> %s" % (uri, e))
             return None
 
-    def copy_from_web_to_store(self, album_id):
-        """
-            Copy artwork from web path to store path
-            @param album_id as int
-        """
-        try:
-            album = Album(album_id)
-            filename = self.get_album_cache_name(album) + ".jpg"
-            web_path = TMP_PATH + "/" + filename
-            store_path = STORE_PATH + "/" + filename
-            web_file = Gio.File.new_for_path(web_path)
-            store_file = Gio.File.new_for_path(store_path)
-            web_file.copy(store_file, Gio.FileCopyFlags.OVERWRITE,
-                          None, None, None)
-        except Exception as e:
-            Logger.error("AlbumArt::copy_from_web_to_store(): %s", e)
-
-    def save_album_artwork(self, data, album):
+    def save_album_artwork(self, album, data):
         """
             Save artwork for album
             @param data as bytes
             @param album as Album
         """
         try:
-            if album.storage_type & StorageType.EPHEMERAL:
-                self.__save_web_album_artwork(data, album)
-            elif album.uri == "" or is_readonly(album.uri):
-                self.__save_ro_album_artwork(data, album)
+            if not album.storage_type & StorageType.COLLECTION:
+                self.__save_web_album_artwork(album, data)
+            elif is_readonly(album.uri):
+                self.__save_ro_album_artwork(album, data)
             else:
-                self.__save_album_artwork(data, album)
+                self.__save_album_artwork(album, data)
         except Exception as e:
             Logger.error("AlbumArt::save_album_artwork(): %s" % e)
 
+    def move_artwork(self, old_lp_album_id, new_lp_album_id):
+        """
+            Move artwork when lp_album_id changed
+            @param old_lp_album_id as str
+            @param new_lp_album_id s str
+        """
+        try:
+            for store in [ALBUMS_WEB_PATH, ALBUMS_PATH]:
+                old_path = "%s/%s.jpg" % (store, old_lp_album_id)
+                old = Gio.File.new_for_path(old_path)
+                if old.query_exists():
+                    new_path = "%s/%s.jpg" % (store, new_lp_album_id)
+                    new = Gio.File.new_for_path(new_path)
+                    old.move(new, Gio.FileCopyFlags.OVERWRITE, None, None)
+                    break
+        except Exception as e:
+            Logger.error("AlbumArt::move_artwork(): %s" % e)
+
     def album_artwork_update(self, album_id):
         """
             Announce album cover update
@@ -307,13 +312,12 @@ class AlbumArt:
         """
         try:
             from pathlib import Path
-            name = self.get_album_cache_name(album)
             if width == -1 or height == -1:
-                for p in Path(CACHE_PATH).glob("%s*.jpg" % name):
+                for p in Path(CACHE_PATH).glob("%s*.jpg" % album.lp_album_id):
                     p.unlink()
             else:
                 filename = "%s/%s_%s_%s.jpg" % (CACHE_PATH,
-                                                name,
+                                                album.lp_album_id,
                                                 width,
                                                 height)
                 f = Gio.File.new_for_path(filename)
@@ -350,16 +354,6 @@ class AlbumArt:
             Logger.error("AlbumArt::pixbuf_from_tags(): %s" % e)
         return pixbuf
 
-    def get_album_cache_name(self, album):
-        """
-            Get a uniq string for album
-            @param album as Album
-        """
-        name = "%s_%s_%s" % (" ".join(album.artists),
-                             album.name,
-                             album.year)
-        return md5(name.encode("utf-8")).hexdigest()
-
 #######################
 # PRIVATE             #
 #######################
@@ -368,7 +362,8 @@ class AlbumArt:
             Check if album uri exists, update if not
             @param album as Album
         """
-        if not album.uri:
+        if not album.storage_type & (StorageType.COLLECTION |
+                                     StorageType.EXTERNAL):
             return
         d = Gio.File.new_for_uri(album.uri)
         if not d.query_exists():
@@ -379,14 +374,13 @@ class AlbumArt:
                 parent_uri = "" if p is None else p.get_uri()
                 album.set_uri(parent_uri)
 
-    def __save_web_album_artwork(self, data, album):
+    def __save_web_album_artwork(self, album, data):
         """
             Save artwork for a web album
-            @param data as bytes
             @param album as Album
+            @param data as bytes
         """
-        filename = self.get_album_cache_name(album) + ".jpg"
-        store_path = TMP_PATH + "/" + filename
+        store_path = "%s/%s.jpg" % (ALBUMS_WEB_PATH, album.lp_album_id)
         if data is None:
             f = Gio.File.new_for_path(store_path)
             fstream = f.replace(None, False,
@@ -397,14 +391,13 @@ class AlbumArt:
         self.clean_album_cache(album)
         self.album_artwork_update(album.id)
 
-    def __save_ro_album_artwork(self, data, album):
+    def __save_ro_album_artwork(self, album, data):
         """
             Save artwork for a read only album
-            @param data as bytes
             @param album as Album
+            @param data as bytes
         """
-        filename = self.get_album_cache_name(album) + ".jpg"
-        store_path = STORE_PATH + "/" + filename
+        store_path = "%s/%s.jpg" % (ALBUMS_PATH, album.lp_album_id)
         if data is None:
             f = Gio.File.new_for_path(store_path)
             fstream = f.replace(None, False,
@@ -415,14 +408,13 @@ class AlbumArt:
         self.clean_album_cache(album)
         self.album_artwork_update(album.id)
 
-    def __save_album_artwork(self, data, album):
+    def __save_album_artwork(self, album, data):
         """
             Save artwork for an album
-            @param data as bytes
             @param album as Album
+            @param data as bytes
         """
-        filename = self.get_album_cache_name(album) + ".jpg"
-        store_path = STORE_PATH + "/" + filename
+        store_path = "%s/%s.jpg" % (ALBUMS_PATH, album.lp_album_id)
         save_to_tags = App().settings.get_value("save-to-tags")
         # Multiple albums at same path
         uri_count = App().albums.get_uri_count(album.uri)
@@ -431,7 +423,7 @@ class AlbumArt:
         # Save cover to tags
         if save_to_tags:
             helper = TaskHelper()
-            helper.run(self.__save_album_artwork_to_tags, data, album)
+            helper.run(self.__save_album_artwork_to_tags, album, data)
 
         # We need to remove favorite if exists
         if uri_count > 1 or save_to_tags:
@@ -441,7 +433,7 @@ class AlbumArt:
 
         # Name file with album information
         if uri_count > 1:
-            art_uri = album.uri + "/" + filename
+            art_uri = "%s/%s.jpg" % (album.uri, album.lp_album_id)
 
         if data is None:
             f = Gio.File.new_for_path(store_path)
@@ -456,11 +448,11 @@ class AlbumArt:
         self.clean_album_cache(album)
         self.album_artwork_update(album.id)
 
-    def __save_album_artwork_to_tags(self, data, album):
+    def __save_album_artwork_to_tags(self, album, data):
         """
             Save artwork to tags
-            @param data as bytes
             @param album as Album
+            @param data as bytes
         """
         # https://bugzilla.gnome.org/show_bug.cgi?id=747431
         bytes = GLib.Bytes.new(data)
