@@ -10,17 +10,18 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import GLib, Gio
+from gi.repository import GLib, Gio, Gtk
 
 import itertools
 from time import time
+from gettext import gettext as _
 
 from lollypop.sqlcursor import SqlCursor
 from lollypop.utils import translate_artist_name
 from lollypop.database_history import History
-from lollypop.radios import Radios
-from lollypop.define import App, Type, StorageType
+from lollypop.define import App, Type, StorageType, LOLLYPOP_DATA_PATH
 from lollypop.logger import Logger
+from lollypop.helper_task import TaskHelper
 
 
 class DatabaseUpgrade:
@@ -43,6 +44,7 @@ class DatabaseUpgrade:
             @param db as Database
         """
         version = 0
+        SqlCursor.add(db)
         with SqlCursor(db, True) as sql:
             result = sql.execute("PRAGMA user_version")
             v = result.fetchone()
@@ -53,12 +55,15 @@ class DatabaseUpgrade:
                     try:
                         if isinstance(self._UPGRADES[i], str):
                             sql.execute(self._UPGRADES[i])
+                            SqlCursor.commit(db)
                         else:
                             self._UPGRADES[i](db)
+                            SqlCursor.commit(db)
                     except Exception as e:
                         Logger.error("DB upgrade %s failed: %s" %
                                      (i, e))
                 sql.execute("PRAGMA user_version=%s" % self.version)
+        SqlCursor.remove(db)
 
     @property
     def version(self):
@@ -156,7 +161,18 @@ class DatabaseAlbumsUpgrade(DatabaseUpgrade):
                                                 mtime INT NOT NULL,
                                                 popularity INT NOT NULL)""",
             39: self.__upgrade_39,
-            40: """UPDATE tracks SET duration = duration * 1000"""
+            40: """UPDATE tracks SET duration = duration * 1000""",
+            # Here we force an mb_album_id if empty, needed by artwork
+            41: """UPDATE albums SET mb_album_id=rowid
+                   WHERE mb_album_id is null""",
+            # Fix previous update
+            42: """UPDATE albums SET mb_album_id=null
+                   WHERE storage_type=2 AND rowid=mb_album_id""",
+            43: """CREATE TABLE featuring (artist_id INT NOT NULL,
+                                           album_id INT NOT NULL)""",
+            44: self.__upgrade_44,
+            45: self.__upgrade_45,
+            46: self.__upgrade_46
         }
 
 #######################
@@ -466,12 +482,7 @@ class DatabaseAlbumsUpgrade(DatabaseUpgrade):
             sql.execute("DROP TABLE backup")
 
     def __upgrade_21(self, db):
-        """
-            Add rate to radios
-        """
-        with SqlCursor(Radios()) as sql:
-            sql.execute("ALTER TABLE radios ADD rate\
-                         INT NOT NULL DEFAULT -1")
+        pass
 
     def __upgrade_22(self, db):
         """
@@ -709,3 +720,99 @@ class DatabaseAlbumsUpgrade(DatabaseUpgrade):
                             FROM album_artists) AND artists.rowid NOT IN (\
                                 SELECT track_artists.artist_id\
                                 FROM track_artists)")
+
+    def __upgrade_44(self, db):
+        """
+            Delete spotify albums as spotify id is not stored in URI
+        """
+        from lollypop.database_albums import AlbumsDatabase
+        from lollypop.database_artists import ArtistsDatabase
+        from lollypop.database_tracks import TracksDatabase
+        albums = AlbumsDatabase(db)
+        artists = ArtistsDatabase(db)
+        tracks = TracksDatabase(db)
+        for storage_type in [StorageType.SPOTIFY_NEW_RELEASES,
+                             StorageType.SPOTIFY_SIMILARS,
+                             StorageType.DEEZER_CHARTS]:
+            album_ids = albums.get_for_storage_type(storage_type)
+            for album_id in album_ids:
+                # EPHEMERAL with not tracks will be cleaned below
+                albums.set_storage_type(album_id, StorageType.EPHEMERAL)
+                tracks.remove_album(album_id)
+        tracks.clean()
+        albums.clean()
+        artists.clean()
+
+    def __upgrade_45(self, db):
+        """
+            Add lp_album_id/lp_track_id
+        """
+        with SqlCursor(db, True) as sql:
+            sql.execute("ALTER TABLE tracks ADD lp_track_id TEXT")
+            sql.execute("ALTER TABLE albums ADD lp_album_id TEXT")
+
+    def __upgrade_46(self, db):
+        """
+            Populate lp_album_id/lp_track_id
+        """
+        queue = LOLLYPOP_DATA_PATH + "/queue.bin"
+        try:
+            f = Gio.File.new_for_path(queue)
+            f.delete(None)
+        except:
+            pass
+        from lollypop.database_albums import AlbumsDatabase
+        from lollypop.database_tracks import TracksDatabase
+        from lollypop.utils import get_lollypop_album_id, get_lollypop_track_id
+        albums = AlbumsDatabase(db)
+        tracks = TracksDatabase(db)
+
+        def do_migration(dialog, label, progress):
+            GLib.idle_add(
+                label.set_text,
+                _("Please wait while Lollypop is updating albums"))
+            album_ids = albums.get_ids([], [], StorageType.ALL, True)
+            album_ids += albums.get_compilation_ids([], StorageType.ALL, True)
+            count = len(album_ids)
+            i = 0
+            for album_id in album_ids:
+                if i % 10 == 0:
+                    GLib.idle_add(progress.set_fraction, i / count)
+                name = albums.get_name(album_id)
+                artists = ";".join(albums.get_artists(album_id))
+                lp_album_id = get_lollypop_album_id(name, artists)
+                albums.set_lp_album_id(album_id, lp_album_id)
+                i += 1
+
+            track_ids = tracks.get_ids(StorageType.ALL, True)
+            count = len(track_ids)
+            i = 0
+            GLib.idle_add(
+                label.set_text,
+                _("Please wait while Lollypop is updating tracks"))
+            for track_id in track_ids:
+                if i % 10 == 0:
+                    GLib.idle_add(progress.set_fraction, i / count)
+                name = tracks.get_name(track_id)
+                artists = ";".join(tracks.get_artists(track_id))
+                album_name = tracks.get_album_name(track_id)
+                lp_track_id = get_lollypop_track_id(name, artists, album_name)
+                tracks.set_lp_track_id(track_id, lp_track_id)
+                i += 1
+            GLib.idle_add(dialog.destroy)
+
+        dialog = Gtk.MessageDialog(buttons=Gtk.ButtonsType.NONE)
+        progress = Gtk.ProgressBar.new()
+        progress.show()
+        label = Gtk.Label.new()
+        label.show()
+        grid = Gtk.Grid.new()
+        grid.set_orientation(Gtk.Orientation.VERTICAL)
+        grid.set_row_spacing(10)
+        grid.show()
+        grid.add(label)
+        grid.add(progress)
+        dialog.set_image(grid)
+        helper = TaskHelper()
+        helper.run(do_migration, dialog, label, progress)
+        dialog.run()
