@@ -13,9 +13,10 @@
 from gi.repository import Gio, GLib
 
 from lollypop.logger import Logger
-from lollypop.objects_radio import Radio
 from lollypop.helper_task import TaskHelper
 from lollypop.utils import escape, get_network_available
+from lollypop.utils_file import create_dir
+from lollypop.define import LYRICS_PATH
 
 
 class LyricsHelper:
@@ -29,6 +30,7 @@ class LyricsHelper:
         """
         self.__timestamps = {}
         self.__cancellable = Gio.Cancellable.new()
+        create_dir(LYRICS_PATH)
 
     def load(self, track):
         """
@@ -170,13 +172,7 @@ class LyricsHelper:
             @return str
         """
         # Update lyrics
-        title = ""
-        if isinstance(track, Radio):
-            split = " ".join(track.artists).split(" - ")
-            if len(split) > 1:
-                title = split[1]
-        else:
-            title = track.name
+        title = track.name
         if escape:
             return GLib.uri_escape_string(title, None, False)
         else:
@@ -189,17 +185,10 @@ class LyricsHelper:
             @param escape as bool
             @return str
         """
-        # Update lyrics
-        artist = ""
-        if isinstance(track, Radio):
-            split = " ".join(track.artists).split(" - ")
-            if len(split) > 0:
-                artist = split[0]
-        else:
-            if track.artists:
-                artist = track.artists[0]
-            elif track.album_artists:
-                artist = track.album_artists[0]
+        if track.artists:
+            artist = track.artists[0]
+        elif track.album_artists:
+            artist = track.album_artists[0]
         if escape:
             return GLib.uri_escape_string(artist, None, False)
         else:
@@ -212,9 +201,10 @@ class LyricsHelper:
             @param methods as []
             @param callback as function
         """
-        title = self.__get_title(track, True)
-        artist = self.__get_artist(track, True)
-        uri = "https://lyrics.wikia.com/wiki/%s:%s" % (artist, title)
+        title = self.__get_title(track, False)
+        artist = self.__get_artist(track, False).lower()
+        string = "%s:%s" % (artist, title)
+        uri = "https://lyrics.wikia.com/wiki/%s" % string.replace(" ", "_")
         helper = TaskHelper()
         helper.load_uri_content(uri,
                                 self.__cancellable,
@@ -235,7 +225,9 @@ class LyricsHelper:
         """
         title = self.__get_title(track, False)
         artist = self.__get_artist(track, False)
-        string = escape("%s %s" % (artist, title)).replace(".", "")
+        string = escape("%s %s" % (artist, title),
+                        ignore=["_", "-", " ", ".", "/"]).replace(".", "")
+        string = string.replace("/", " ")
         uri = "https://genius.com/%s-lyrics" % string.replace(" ", "-")
         helper = TaskHelper()
         helper.load_uri_content(uri,
