@@ -18,7 +18,7 @@ from gettext import gettext as _
 from lollypop.define import StorageType, MARGIN_SMALL, App, CACHE_PATH
 from lollypop.define import ViewType
 from lollypop.menu_playlists import PlaylistsMenu
-from lollypop.menu_edit import EditMenu
+from lollypop.menu_actions import ActionsMenu
 from lollypop.menu_playback import TrackPlaybackMenu, AlbumPlaybackMenu
 from lollypop.menu_sync import SyncAlbumsMenu
 from lollypop.widgets_rating import RatingWidget
@@ -30,10 +30,11 @@ class AlbumMenu(Gio.Menu):
         Contextual menu for album
     """
 
-    def __init__(self, album, view_type, header=False):
+    def __init__(self, album, storage_type, view_type, header=False):
         """
             Init menu model
             @param album as Album
+            @param storage_type as StorageType
             @param view_type as ViewType
             @param header as bool
         """
@@ -41,41 +42,40 @@ class AlbumMenu(Gio.Menu):
         if header:
             from lollypop.menu_header import AlbumMenuHeader
             self.append_item(AlbumMenuHeader(album))
-        if not view_type & ViewType.BANNER:
-            self.append_section(_("Playback"), AlbumPlaybackMenu(album))
+        self.append_section(_("Playback"),
+                            AlbumPlaybackMenu(album, view_type))
         from lollypop.menu_artist import ArtistAlbumsMenu
-        menu = ArtistAlbumsMenu(album.artist_ids[0], view_type)
+        menu = ArtistAlbumsMenu(album.artist_ids[0], album.storage_type)
         if menu.get_n_items() != 0:
             self.append_section(_("Artist"), menu)
         section = Gio.Menu()
         if album.storage_type & (StorageType.COLLECTION | StorageType.SAVED):
-            section.append_submenu(_("Playlists"), PlaylistsMenu(album))
+            section.append_submenu(_("Playlists"), PlaylistsMenu([album]))
         if album.storage_type & StorageType.COLLECTION:
             section.append_submenu(_("Devices"), SyncAlbumsMenu([album]))
         if section.get_n_items() != 0:
             self.append_section(_("Add to"), section)
-        self.append_section(_("Edit"), EditMenu(album))
+        self.append_section(_("Others"), ActionsMenu(album))
 
 
-class MinTrackMenu(Gio.Menu):
+class AlbumsMenu(Gio.Menu):
     """
-        Contextual menu for a track
+        Contextual menu for albums
     """
 
-    def __init__(self, track):
+    def __init__(self, title, albums, header=False):
         """
             Init menu model
-            @param track as Track
+            @param title as str
+            @param albums as [Album]
+            @param header as bool
         """
         Gio.Menu.__init__(self)
-        from lollypop.menu_artist import ArtistAlbumsMenu
-        menu = ArtistAlbumsMenu(track.artist_ids[0], ViewType.DEFAULT)
-        self.append_section(_("Artist"), menu)
-        if not track.storage_type & StorageType.EPHEMERAL:
-            section = Gio.Menu()
-            section.append_submenu(_("Playlists"), PlaylistsMenu(track))
-            self.append_section(_("Add to"), section)
-        self.append_section(_("Edit"), EditMenu(track))
+        if header:
+            from lollypop.menu_header import MenuHeader
+            self.append_item(MenuHeader(title,
+                                        "media-optical-cd-audio-symbolic"))
+        self.append_section(_("Devices"), SyncAlbumsMenu(albums))
 
 
 class TrackMenu(Gio.Menu):
@@ -83,22 +83,28 @@ class TrackMenu(Gio.Menu):
         Full Contextual menu for a track
     """
 
-    def __init__(self, track, header):
+    def __init__(self, track, view_type, header):
         """
             Init menu model
             @param track as Track
+            @param view_type as ViewType
             @param header as bool
         """
         Gio.Menu.__init__(self)
         if header:
             from lollypop.menu_header import AlbumMenuHeader
             self.append_item(AlbumMenuHeader(track))
-        self.append_section(_("Playback"), TrackPlaybackMenu(track))
+        self.append_section(_("Playback"), TrackPlaybackMenu(track, view_type))
+        if view_type & ViewType.SEARCH:
+            from lollypop.menu_artist import ArtistAlbumsMenu
+            menu = ArtistAlbumsMenu(track.artist_ids[0], track.storage_type)
+            if menu.get_n_items() != 0:
+                self.append_section(_("Artist"), menu)
         if not track.storage_type & StorageType.EPHEMERAL:
             section = Gio.Menu()
-            section.append_submenu(_("Playlists"), PlaylistsMenu(track))
+            section.append_submenu(_("Playlists"), PlaylistsMenu([track]))
             self.append_section(_("Add to"), section)
-        self.append_section(_("Edit"), EditMenu(track))
+        self.append_section(_("Others"), ActionsMenu(track))
 
 
 class TrackMenuExt(Gtk.Grid):
@@ -169,6 +175,10 @@ class TrackMenuExt(Gtk.Grid):
         self.add(separator)
         self.add(hgrid)
 
+    @property
+    def submenu_name(self):
+        return None
+
 #######################
 # PRIVATE             #
 #######################
