@@ -13,6 +13,7 @@
 from gi.repository import Gst, GLib, GObject
 
 from pickle import load
+from time import time
 
 from lollypop.player_albums import AlbumsPlayer
 from lollypop.player_auto_random import AutoRandomPlayer
@@ -21,18 +22,15 @@ from lollypop.player_bin import BinPlayer
 from lollypop.player_queue import QueuePlayer
 from lollypop.player_linear import LinearPlayer
 from lollypop.player_shuffle import ShufflePlayer
-from lollypop.player_radio import RadioPlayer
 from lollypop.player_transitions import TransitionsPlayer
-from lollypop.radios import Radios
 from lollypop.logger import Logger
 from lollypop.objects_track import Track
-from lollypop.objects_radio import Radio
 from lollypop.define import App, Type, LOLLYPOP_DATA_PATH
 from lollypop.utils import emit_signal
 
 
 class Player(GObject.GObject, AlbumsPlayer, BinPlayer, AutoRandomPlayer,
-             AutoSimilarPlayer, QueuePlayer, RadioPlayer, LinearPlayer,
+             AutoSimilarPlayer, QueuePlayer, LinearPlayer,
              ShufflePlayer, TransitionsPlayer):
     """
         Player object used to manage playback and playlists
@@ -49,7 +47,14 @@ class Player(GObject.GObject, AlbumsPlayer, BinPlayer, AutoRandomPlayer,
         "status-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
         "volume-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
         "queue-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
-        "playback-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "playback-added": (GObject.SignalFlags.RUN_FIRST, None,
+                           (GObject.TYPE_PYOBJECT,)),
+        "playback-updated": (GObject.SignalFlags.RUN_FIRST, None,
+                             (GObject.TYPE_PYOBJECT,)),
+        "playback-setted": (GObject.SignalFlags.RUN_FIRST, None,
+                            (GObject.TYPE_PYOBJECT,)),
+        "playback-removed": (GObject.SignalFlags.RUN_FIRST, None,
+                             (GObject.TYPE_PYOBJECT,)),
         "rate-changed": (GObject.SignalFlags.RUN_FIRST, None, (int, int))
     }
 
@@ -65,7 +70,6 @@ class Player(GObject.GObject, AlbumsPlayer, BinPlayer, AutoRandomPlayer,
         QueuePlayer.__init__(self)
         LinearPlayer.__init__(self)
         ShufflePlayer.__init__(self)
-        RadioPlayer.__init__(self)
         TransitionsPlayer.__init__(self)
         self.__stop_after_track_id = None
         App().settings.connect("changed::repeat", self.update_next_prev)
@@ -75,9 +79,7 @@ class Player(GObject.GObject, AlbumsPlayer, BinPlayer, AutoRandomPlayer,
             Stop current track, load track id and play it
             @param track as Track
         """
-        if isinstance(track, Radio):
-            RadioPlayer.load(self, track)
-        elif TransitionsPlayer.load(self, track):
+        if TransitionsPlayer.load(self, track):
             pass
         else:
             BinPlayer.load(self, track)
@@ -100,8 +102,8 @@ class Player(GObject.GObject, AlbumsPlayer, BinPlayer, AutoRandomPlayer,
         """
             Play next track
         """
+        self._on_track_finished(self._current_track)
         if self._next_track.id is not None:
-            self._scrobble(self._current_track, self._start_time)
             self.load(self._next_track)
         else:
             self.stop()
@@ -131,37 +133,25 @@ class Player(GObject.GObject, AlbumsPlayer, BinPlayer, AutoRandomPlayer,
         """
         try:
             if App().settings.get_value("save-state"):
-                self._current_playback_track = Track(
+                self._current_track = Track(
                     load(open(LOLLYPOP_DATA_PATH + "/track_id.bin", "rb")))
                 self.set_queue(load(open(LOLLYPOP_DATA_PATH +
                                          "/queue.bin", "rb")))
                 albums = load(open(LOLLYPOP_DATA_PATH + "/Albums.bin", "rb"))
-                playlist_ids = load(open(LOLLYPOP_DATA_PATH +
-                                         "/playlist_ids.bin", "rb"))
                 (is_playing, was_party) = load(open(LOLLYPOP_DATA_PATH +
                                                     "/player.bin", "rb"))
-                if playlist_ids and playlist_ids[0] == Type.RADIOS:
-                    radios = Radios()
-                    track = Track()
-                    name = radios.get_name(self._current_playback_track.id)
-                    uri = radios.get_uri(self._current_playback_track.id)
-                    track.set_radio(name, uri)
-                    self.load(track)
-                elif self._current_playback_track.uri:
+                if self._current_track.uri:
                     if albums:
                         if was_party:
                             App().lookup_action("party").change_state(
                                 GLib.Variant("b", True))
                         else:
-                            self._albums = load(open(
-                                                LOLLYPOP_DATA_PATH +
-                                                "/Albums.bin",
-                                                "rb"))
+                            self.set_albums(albums)
                         # Load track from player albums
                         index = self.album_ids.index(
-                            self._current_playback_track.album.id)
+                            self._current_track.album.id)
                         for track in self._albums[index].tracks:
-                            if track.id == self._current_playback_track.id:
+                            if track.id == self._current_track.id:
                                 self._load_track(track)
                                 break
                     if is_playing:
@@ -172,7 +162,7 @@ class Player(GObject.GObject, AlbumsPlayer, BinPlayer, AutoRandomPlayer,
                                     "rb"))
                     self.seek(position)
                 else:
-                    Logger.info("Player::restore_state(): track missing")
+                    Logger.debug("Player::restore_state(): track missing")
         except Exception as e:
             Logger.error("Player::restore_state(): %s" % e)
 
@@ -189,7 +179,7 @@ class Player(GObject.GObject, AlbumsPlayer, BinPlayer, AutoRandomPlayer,
         """
             Set previous track
         """
-        if isinstance(self.current_track, Radio):
+        if self._current_track.id is None:
             return
         try:
             if App().settings.get_value("shuffle") or self.is_party:
@@ -205,8 +195,9 @@ class Player(GObject.GObject, AlbumsPlayer, BinPlayer, AutoRandomPlayer,
         """
             Play next track
         """
-        if isinstance(self.current_track, Radio) or\
-                self._current_track.id == self.__stop_after_track_id:
+        if self._current_track.id is None:
+            return
+        if self._current_track.id == self.__stop_after_track_id:
             self._next_track = Track()
             return
         try:
@@ -217,8 +208,6 @@ class Player(GObject.GObject, AlbumsPlayer, BinPlayer, AutoRandomPlayer,
                 else:
                     next_track = LinearPlayer.next(self)
             self._next_track = next_track
-            if next_track.is_web:
-                App().task_helper.run(self._load_from_web, next_track, False)
             emit_signal(self, "next-changed")
         except Exception as e:
             Logger.error("Player::set_next(): %s" % e)
@@ -258,20 +247,54 @@ class Player(GObject.GObject, AlbumsPlayer, BinPlayer, AutoRandomPlayer,
 #######################
 # PROTECTED           #
 #######################
+    def _on_track_finished(self, track):
+        """
+            Scrobble track, update last played time and increment popularity
+            @param track as Track
+        """
+        if track.id is None:
+            return
+        # Track has been played
+        # for at least half its duration, or for 4 minutes
+        played = time() - self._start_time
+        if played >= track.duration / 2000 or played >= 240:
+            self.__scrobble(track, self._start_time)
+            if track.id >= 0:
+                App().tracks.set_listened_at(track.id, int(time()))
+                # Increment popularity
+                App().tracks.set_more_popular(track.id)
+                # In party mode, linear popularity
+                if self.is_party:
+                    pop_to_add = 1
+                # In normal mode, based on tracks count
+                else:
+                    count = track.album.tracks_count
+                    pop_to_add = int(App().albums.max_count / count)
+                App().albums.set_more_popular(track.album_id, pop_to_add)
+
     def _on_stream_start(self, bus, message):
         """
             On stream start, set next and previous track
         """
-        self.__stop_after_track_id = None
         if self.is_in_queue(self._current_track.id):
             self.remove_from_queue(self._current_track.id)
-        else:
-            self._current_playback_track = self._current_track
         ShufflePlayer._on_stream_start(self, bus, message)
         BinPlayer._on_stream_start(self, bus, message)
+        AutoSimilarPlayer._on_stream_start(self, bus, message)
         self.set_next()
         self.set_prev()
 
 #######################
 # PRIVATE             #
 #######################
+    def __scrobble(self, track, finished_start_time):
+        """
+            Scrobble on lastfm
+            @param track as Track
+            @param finished_start_time as int
+        """
+        # Last.fm policy, force it for ListenBrainz too
+        if track.duration < 30000:
+            return
+        for scrobbler in App().ws_director.scrobblers:
+            scrobbler.listen(track, int(finished_start_time))
