@@ -14,10 +14,9 @@ from random import shuffle, random
 
 from lollypop.define import Repeat, App
 from lollypop.objects_track import Track
-from lollypop.objects_radio import Radio
 from lollypop.objects_album import Album
 from lollypop.list import LinkedList
-from lollypop.utils import emit_signal
+from lollypop.utils import emit_signal, get_default_storage_type
 from lollypop.logger import Logger
 
 
@@ -42,7 +41,9 @@ class ShufflePlayer:
         # Party mode
         self.__is_party = False
         App().settings.connect("changed::shuffle", self.__set_shuffle)
-        self.connect("playback-changed", self.__on_playback_changed)
+        self.connect("playback-added", self.__on_playback_added)
+        self.connect("playback-setted", self.__on_playback_setted)
+        self.connect("playback-removed", self.__on_playback_removed)
 
     def next(self):
         """
@@ -51,7 +52,7 @@ class ShufflePlayer:
         """
         repeat = App().settings.get_enum("repeat")
         if repeat == Repeat.TRACK:
-            return self._current_playback_track
+            return self._current_track
         if self.shuffle_has_next:
             track = self.__history.next.value
         elif self._albums:
@@ -67,11 +68,11 @@ class ShufflePlayer:
         """
         repeat = App().settings.get_enum("repeat")
         if repeat == Repeat.TRACK:
-            return self._current_playback_track
+            return self._current_track
         if self.shuffle_has_prev:
             track = self.__history.prev.value
         else:
-            track = self._current_playback_track
+            track = self._current_track
         return track
 
     def set_party(self, party):
@@ -80,25 +81,29 @@ class ShufflePlayer:
             Play a new random track if not already playing
             @param party as bool
         """
+        def start_party(*ignore):
+            if self._albums:
+                # Start a new song if not playing
+                if self._current_track.id is None:
+                    track = self.__get_tracks_random()
+                    self.load(track)
+                elif not self.is_playing:
+                    self.play()
+            emit_signal(self, "loading-changed", False, Track())
+
         if party == self.__is_party:
             return
         self.__is_party = party
 
         if party:
-            self.set_party_ids()
-            if self._albums:
-                # Start a new song if not playing
-                if self._current_playback_track.id is None or\
-                        isinstance(self._current_playback_track, Radio):
-                    track = self.__get_tracks_random()
-                    self.load(track)
-                elif not self.is_playing:
-                    self.play()
+            App().task_helper.run(self.set_party_ids, callback=(start_party,))
         else:
             # We want current album to continue playback
-            self._albums = [self._current_playback_track.album]
-            emit_signal(self, "playback-changed")
-        if self._current_playback_track.id is not None:
+            self._albums = [self._current_track.album]
+            emit_signal(self, "playback-setted", [])
+            emit_signal(self, "playback-added",
+                        self._current_track.album)
+        if self._current_track.id is not None:
             self.set_next()
             self.set_prev()
 
@@ -106,17 +111,19 @@ class ShufflePlayer:
         """
             Set party mode ids
         """
+        if not self.__is_party:
+            return
         party_ids = App().settings.get_value("party-ids")
-        album_ids = App().albums.get_ids([], party_ids, True)
-        # All albums are blacklisted, WTF?
-        if not album_ids:
-            album_ids = App().albums.get_ids([], party_ids, False)
+        storage_type = get_default_storage_type()
+        album_ids = App().albums.get_ids(party_ids, [], storage_type, False)
+        emit_signal(self, "playback-setted", [])
         self._albums = []
+        if album_ids:
+            emit_signal(self, "loading-changed", True, Track())
         for album_id in album_ids:
-            album = Album(album_id, [], [])
-            album.set_skipping_allowed()
+            album = Album(album_id, [], [], False)
             self._albums.append(album)
-        emit_signal(self, "playback-changed")
+        emit_signal(self, "playback-setted", list(self._albums))
 
     @property
     def is_party(self):
@@ -149,35 +156,36 @@ class ShufflePlayer:
         """
             On stream start add to shuffle history
         """
-        if self._current_playback_track.id is None or\
-                self._current_playback_track.id < 0:
+        if self._current_track.id is None or\
+                self._current_track.id < 0:
             return
         # Add track to shuffle history if needed
         if App().settings.get_value("shuffle") or self.__is_party:
+            self.__add_to_shuffle_history(self._current_track)
             if self.__history:
                 next = self.__history.next
                 prev = self.__history.prev
                 # Remove next track
                 if next is not None and\
-                        self._current_playback_track == next.value:
+                        self._current_track == next.value:
                     next = self.__history.next
                     next.set_prev(self.__history)
                     self.__history = next
                 # Remove previous track
                 elif prev is not None and\
-                        self._current_playback_track == prev.value:
+                        self._current_track == prev.value:
                     prev = self.__history.prev
                     prev.set_next(self.__history)
                     self.__history = prev
                 # Add a new track
-                elif self.__history.value != self._current_playback_track:
-                    new_list = LinkedList(self._current_playback_track,
+                elif self.__history.value != self._current_track:
+                    new_list = LinkedList(self._current_track,
                                           None,
                                           self.__history)
                     self.__history = new_list
             else:
                 # Initial history
-                new_list = LinkedList(self._current_playback_track)
+                new_list = LinkedList(self._current_track)
                 self.__history = new_list
 
 #######################
@@ -189,7 +197,8 @@ class ShufflePlayer:
             @param settings as Gio.Settings
             @param value as GLib.Variant
         """
-        if self._current_playback_track.id is not None:
+        self.__on_playback_setted(self, self.albums)
+        if self._current_track.id is not None:
             self.set_next()
 
     def __get_next(self):
@@ -201,19 +210,22 @@ class ShufflePlayer:
             if App().settings.get_value("shuffle") or self.__is_party:
                 if self._albums:
                     track = self.__get_tracks_random()
-                    # All track dones
+                    # All tracks done
                     # Try to get another one track after reseting history
                     if track.id is None:
                         self.__to_play_albums = list(self._albums)
                         shuffle(self.__to_play_albums)
-                        self.__not_played_albums = list(
-                            self.__not_played_albums)
-                        self.__history = []
                         repeat = App().settings.get_enum("repeat")
+                        # Do not reset history if a new album is going to
+                        # be added
+                        if repeat not in [Repeat.AUTO_SIMILAR,
+                                          Repeat.AUTO_RANDOM]:
+                            self.__history = []
+                            self.__already_played_tracks = {}
                         if repeat == Repeat.ALL:
                             return self.__get_next()
                     return track
-        except Exception as e:  # Recursion error
+        except Exception as e:
             Logger.error("ShufflePLayer::__get_next(): %s", e)
         return Track()
 
@@ -227,10 +239,11 @@ class ShufflePlayer:
             self.__not_played_albums = list(self.__to_play_albums)
         while self.__not_played_albums:
             album = self.__not_played_albums.pop(0)
+            if not album.tracks:
+                continue
             for track in sorted(album.tracks,
                                 key=lambda *args: random()):
                 if not self.__in_shuffle_history(track):
-                    self.__add_to_shuffle_history(track)
                     return track
             self.__to_play_albums.remove(album)
         if self.__to_play_albums:
@@ -257,15 +270,47 @@ class ShufflePlayer:
         if track not in self.__already_played_tracks[track.album.id]:
             self.__already_played_tracks[track.album.id].append(track.id)
 
-    def __on_playback_changed(self, player):
+    def __on_playback_added(self, player, album):
         """
-            Reset history
+            Update shuffle for album
             @param player as Player
+            @param album as Album
         """
-        self.__to_play_albums = list(self._albums)
-        shuffle(self.__to_play_albums)
-        self.__not_played_albums = list(self.__to_play_albums)
-        self.__history = []
-        self.__already_played_tracks = {}
-        if App().player.current_track.id is not None:
-            self.__add_to_shuffle_history(App().player.current_track)
+        if App().settings.get_value("shuffle") or self.__is_party:
+            if album not in self.__to_play_albums:
+                self.__to_play_albums.append(album)
+                shuffle(self.__to_play_albums)
+            if album not in self.__not_played_albums:
+                self.__not_played_albums.append(album)
+            # If album already playing or
+            # if current track was last one
+            if App().player.current_track.album == album or\
+                    not self.__already_played_tracks:
+                self.__add_to_shuffle_history(App().player.current_track)
+
+    def __on_playback_setted(self, player, albums):
+        """
+            Update shuffle for album
+            @param player as Player
+            @param albums as [Album]
+        """
+        if App().settings.get_value("shuffle") or self.__is_party:
+            self.__to_play_albums = albums
+            if albums:
+                shuffle(self.__to_play_albums)
+            self.__not_played_albums = []
+            self.__already_played_tracks = {}
+            if App().player.current_track.album in albums:
+                self.__add_to_shuffle_history(App().player.current_track)
+
+    def __on_playback_removed(self, player, album):
+        """
+            Update shuffle for album
+            @param player as Player
+            @param album as Album
+        """
+        if App().settings.get_value("shuffle") or self.__is_party:
+            if album in self.__to_play_albums:
+                self.__to_play_albums.remove(album)
+            if album in self.__not_played_albums:
+                self.__not_played_albums.remove(album)
