@@ -10,82 +10,74 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, GLib, GObject
+from gi.repository import Gtk
 
 from lollypop.define import App, ViewType, MARGIN, ScanUpdate
 from lollypop.view_tracks_album import AlbumTracksView
 from lollypop.widgets_banner_album import AlbumBannerWidget
-from lollypop.controller_view import ViewController, ViewControllerType
 from lollypop.view_lazyloading import LazyLoadingView
-from lollypop.helper_filtering import FilteringHelper
 from lollypop.helper_signals import SignalsHelper, signals_map
-from lollypop.logger import Logger
 
 
-class AlbumView(FilteringHelper, LazyLoadingView,
-                ViewController, SignalsHelper):
+class AlbumView(LazyLoadingView, SignalsHelper):
     """
         Show artist albums and tracks
     """
 
-    __gsignals__ = {
-        "populated": (GObject.SignalFlags.RUN_FIRST, None, ()),
-    }
-
     @signals_map
-    def __init__(self, album, view_type):
+    def __init__(self, album, storage_type, view_type):
         """
             Init ArtistView
             @param album as Album
+            @param storage_type as StorageType
             @param view_type as ViewType
         """
-        LazyLoadingView.__init__(self, view_type)
-        ViewController.__init__(self, ViewControllerType.ALBUM)
-        FilteringHelper.__init__(self)
+        LazyLoadingView.__init__(self, storage_type, view_type)
+        self.__tracks_view = None
         self.__album = album
         self.__others_boxes = []
         self.__grid = Gtk.Grid()
         self.__grid.show()
         self.__grid.set_row_spacing(10)
         self.__grid.set_orientation(Gtk.Orientation.VERTICAL)
-        self.__banner = AlbumBannerWidget(self.__album, self._view_type)
+        self.__banner = AlbumBannerWidget(self.__album, self.storage_type,
+                                          self.view_type)
         self.add_widget(self.__grid, self.__banner)
         return [
             (App().scanner, "scan-finished", "_on_scan_finished"),
-            (App().scanner, "album-updated", "_on_album_updated")
+            (App().scanner, "updated", "_on_collection_updated"),
+            (App().player, "current-changed", "_on_current_changed"),
+            (App().player, "duration-changed", "_on_duration_changed"),
         ]
 
     def populate(self):
         """
             Populate the view with album
         """
-        self.__tracks_view = AlbumTracksView(self.__album, ViewType.ALBUM)
-        self.__tracks_view.show()
-        self.__tracks_view.connect("populated", self.__on_tracks_populated)
-        self.__tracks_view.set_margin_start(MARGIN)
-        self.__tracks_view.set_margin_end(MARGIN)
-        self.__tracks_view.populate()
-        self.__grid.add(self.__tracks_view)
+        if self.__tracks_view is None:
+            self.__tracks_view = AlbumTracksView(self.__album, ViewType.ALBUM)
+            self.__tracks_view.show()
+            self.__tracks_view.connect("populated", self.__on_tracks_populated)
+            self.__tracks_view.set_margin_start(MARGIN)
+            self.__tracks_view.set_margin_end(MARGIN)
+            self.__tracks_view.populate()
+            self.__grid.add(self.__tracks_view)
 
-    def activate_child(self):
+    @property
+    def name(self):
         """
-            Activated typeahead row
+            Get name
+            @return str
         """
-        try:
-            if App().player.is_party:
-                App().lookup_action("party").change_state(
-                    GLib.Variant("b", False))
-            for child in self.filtered:
-                style_context = child.get_style_context()
-                if style_context.has_class("typeahead"):
-                    if hasattr(child, "data"):
-                        child.activate()
-                    else:
-                        track = child.track
-                        App().player.add_album(track.album)
-                        App().player.load(track.album.get_track(track.id))
-        except Exception as e:
-            Logger.error("AlbumView::activate_child: %s" % e)
+        return self.__album.name
+
+    @property
+    def is_populated(self):
+        """
+            True if populated
+            @return bool
+        """
+        return self.__tracks_view is not None
 
     @property
     def args(self):
@@ -93,7 +85,10 @@ class AlbumView(FilteringHelper, LazyLoadingView,
             Get default args for __class__
             @return {}
         """
-        return {"album": self.__album, "view_type": self.view_type}
+        return {"album": self.__album,
+                "storage_type": self.storage_type,
+                "view_type": self.view_type & ~(ViewType.ADAPTIVE |
+                                                ViewType.SMALL)}
 
     @property
     def filtered(self):
@@ -101,6 +96,8 @@ class AlbumView(FilteringHelper, LazyLoadingView,
             Get filtered children
             @return [Gtk.Widget]
         """
+        if self.__tracks_view is None:
+            return []
         filtered = self.__tracks_view.children
         for box in self.__others_boxes:
             for child in box.children:
@@ -110,47 +107,31 @@ class AlbumView(FilteringHelper, LazyLoadingView,
     @property
     def scroll_shift(self):
         """
-            Add scroll shift on y axes
+            Get scroll shift for y axes
             @return int
         """
         return self.__banner.height
 
-    @property
-    def scroll_relative_to(self):
-        """
-            Relative to scrolled widget
-            @return Gtk.Widget
-        """
-        return self.__tracks_view
-
-    @property
-    def is_populated(self):
-        """
-            True if populated
-            @return bool
-        """
-        return True
-
 #######################
 # PROTECTED           #
 #######################
-    def _on_scan_finished(self, scanner, modifications):
+    def _on_scan_finished(self, scanner, track_ids):
         """
             Reload album if needed
             @param scanner as CollectionScanner
-            @param modifications as bool
+            @param track_ids as int
         """
         if not self.get_sensitive():
             App().window.container.reload_view()
 
-    def _on_album_updated(self, scanner, album_id, scan_update):
+    def _on_collection_updated(self, scanner, item, scan_update):
         """
             Handles changes in collection
             @param scanner as CollectionScanner
-            @param album_id as int
+            @param item as CollectionItem
             @param scan_update as ScanUpdate
         """
-        if album_id != self.__album.id:
+        if item.album_id != self.__album.id:
             return
         if scan_update == ScanUpdate.REMOVED:
             App().window.container.go_back()
@@ -160,7 +141,8 @@ class AlbumView(FilteringHelper, LazyLoadingView,
             Update children state
             @param player as Player
         """
-        self.__tracks_view.set_playing_indicator()
+        if self.__tracks_view is not None:
+            self.__tracks_view.set_playing_indicator()
 
     def _on_duration_changed(self, player, track_id):
         """
@@ -168,7 +150,8 @@ class AlbumView(FilteringHelper, LazyLoadingView,
             @param player as Player
             @param track_id as int
         """
-        self.__tracks_view.update_duration(track_id)
+        if self.__tracks_view is not None:
+            self.__tracks_view.update_duration(track_id)
 
 #######################
 # PRIVATE             #
@@ -181,16 +164,18 @@ class AlbumView(FilteringHelper, LazyLoadingView,
         if self.__tracks_view.is_populated:
             self.emit("populated")
             self.__banner.show()
-            if self._view_type & ViewType.OVERLAY:
+            if self.view_type & ViewType.OVERLAY:
                 from lollypop.view_albums_line import AlbumsArtistLineView
                 for artist_id in self.__album.artist_ids:
-                    others_box = AlbumsArtistLineView(self.__album, artist_id,
+                    others_box = AlbumsArtistLineView(artist_id,
+                                                      self.__album.genre_ids,
+                                                      self.storage_type,
                                                       ViewType.SMALL |
                                                       ViewType.ALBUM |
                                                       ViewType.SCROLLED)
                     others_box.set_margin_start(MARGIN)
                     others_box.set_margin_end(MARGIN)
-                    others_box.populate()
+                    others_box.populate(self.__album.id)
                     self.__grid.add(others_box)
                     self.__others_boxes.append(others_box)
         else:
