@@ -10,16 +10,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gio
-
-from gettext import gettext as _
 from random import shuffle
 
 from lollypop.view_flowbox import FlowBoxView
-from lollypop.define import App, Type, ViewType, OrderBy
+from lollypop.define import App, Type, ViewType, OrderBy, ScanUpdate
 from lollypop.widgets_artist_rounded import RoundedArtistWidget
 from lollypop.objects_album import Album
-from lollypop.utils import get_icon_name, get_font_height
+from lollypop.utils import get_icon_name
 from lollypop.helper_signals import SignalsHelper, signals_map
 
 
@@ -29,18 +26,19 @@ class RoundedArtistsView(FlowBoxView, SignalsHelper):
     """
 
     @signals_map
-    def __init__(self, view_type):
+    def __init__(self, storage_type, view_type):
         """
             Init artist view
+            @param storage_type as StorageType
             @param view_type as ViewType
         """
-        FlowBoxView.__init__(self, view_type)
+        FlowBoxView.__init__(self, storage_type, view_type)
         self.connect("destroy", self.__on_destroy)
         self._empty_icon_name = get_icon_name(Type.ARTISTS)
         return [
             (App().art, "artist-artwork-changed",
              "_on_artist_artwork_changed"),
-            (App().scanner, "artist-updated", "_on_artist_updated")
+            (App().scanner, "updated", "_on_collection_updated")
         ]
 
     def populate(self, artist_ids=[]):
@@ -53,11 +51,7 @@ class RoundedArtistsView(FlowBoxView, SignalsHelper):
             FlowBoxView.populate(self, artist_ids)
 
         def load():
-            if App().settings.get_value("show-performers"):
-                ids = App().artists.get_performers()
-            else:
-                ids = App().artists.get()
-            return ids
+            return App().artists.get([], self.storage_type)
 
         if artist_ids:
             FlowBoxView.populate(self, artist_ids)
@@ -70,20 +64,21 @@ class RoundedArtistsView(FlowBoxView, SignalsHelper):
             Get default args for __class__
             @return {}
         """
-        return {"view_type": self.view_type}
+        return {"storage_type": self.storage_type,
+                "view_type": self.view_type}
 
 #######################
 # PROTECTED           #
 #######################
-    def _get_child(self, value):
+    def _get_child(self, item):
         """
             Get a child for view
-            @param value as object
+            @param item as (int, str, str)
             @return row as SelectionListRow
         """
         if self.destroyed:
             return None
-        widget = RoundedArtistWidget(value, self._view_type, self.font_height)
+        widget = RoundedArtistWidget(item, self.view_type, self.font_height)
         self._box.insert(widget, -1)
         widget.show()
         return widget
@@ -97,20 +92,13 @@ class RoundedArtistsView(FlowBoxView, SignalsHelper):
         from lollypop.widgets_menu import MenuBuilder
         from lollypop.menu_artist import ArtistMenu
         from lollypop.menu_similars import SimilarsMenu
-        menu = ArtistMenu(child.data, self._view_type,
+        menu = ArtistMenu(child.data, self.storage_type, self.view_type,
                           App().window.is_adaptive)
         menu_widget = MenuBuilder(menu, False)
         menu_widget.show()
-        section = Gio.Menu()
-        menu.append_section(_("Similar artists"), section)
-        menu_widget = MenuBuilder(menu, True)
-        menu_widget.show()
-        menu_ext = SimilarsMenu()
+        menu_ext = SimilarsMenu(child.data)
         menu_ext.show()
-        menu_ext.populate(child.data)
         menu_widget.append_widget(menu_ext)
-        scrolled = menu_widget.get_child_by_name("main")
-        scrolled.set_size_request(300, 400)
         return menu_widget
 
     def _on_child_activated(self, flowbox, child):
@@ -119,7 +107,8 @@ class RoundedArtistsView(FlowBoxView, SignalsHelper):
             @param flowbox as Gtk.FlowBox
             @param child as Gtk.FlowBoxChild
         """
-        App().window.container.show_view([Type.ARTISTS], [child.data])
+        App().window.container.show_view([Type.ARTISTS], [child.data],
+                                         self.storage_type)
 
     def _on_tertiary_press_gesture(self, x, y, event):
         """
@@ -131,7 +120,8 @@ class RoundedArtistsView(FlowBoxView, SignalsHelper):
         child = self._box.get_child_at_pos(x, y)
         if child is None or child.artwork is None:
             return
-        album_ids = App().albums.get_ids([child.data], [])
+        album_ids = App().albums.get_ids([], [child.data],
+                                         self.storage_type, False)
         albums = [Album(album_id) for album_id in album_ids]
         if albums:
             App().player.play_album_for_albums(albums[0], albums)
@@ -146,32 +136,23 @@ class RoundedArtistsView(FlowBoxView, SignalsHelper):
             if child.name == prefix:
                 child.set_artwork()
 
-    def _on_artist_updated(self, scanner, artist_id, add):
+    def _on_collection_updated(self, scanner, item, scan_update):
         """
             Add/remove artist to/from list
             @param scanner as CollectionScanner
-            @param artist_id as int
-            @param add as bool
+            @param item as CollectionItem
+            @param scan_update as ScanUpdate
         """
-        if add:
-            artist_ids = App().artists.get_ids()
-            # Can happen during scan
-            if artist_id not in artist_ids:
-                return
-            position = artist_ids.index(artist_id)
-            artist_name = App().artists.get_name(artist_id)
-            sortname = App().artists.get_sortname(artist_id)
-            widget = RoundedArtistWidget((artist_id, artist_name, sortname),
-                                         self._view_type,
-                                         get_font_height())
-            self._box.insert(widget, position)
-            widget.show()
-            widget.populate()
-        else:
-            for child in self._box.get_children():
-                if child.data == artist_id:
-                    child.destroy()
-                    break
+        for artist_id in item.new_album_artist_ids:
+            if scan_update == ScanUpdate.ADDED:
+                artist_name = App().artists.get_name(artist_id)
+                sortname = App().artists.get_sortname(artist_id)
+                self.add_value((artist_id, artist_name, sortname))
+            elif scan_update == ScanUpdate.REMOVED:
+                for child in self._box.get_children():
+                    if child.data == artist_id:
+                        child.destroy()
+                        break
 
 #######################
 # PRIVATE             #
@@ -189,24 +170,31 @@ class RoundedArtistsViewWithBanner(RoundedArtistsView):
         Show rounded artist view with a banner
     """
 
-    def __init__(self):
+    def __init__(self, storage_type, view_type):
         """
             Init artist view
+            @param storage_type as StorageType
+            @param view_type as ViewType
         """
-        from lollypop.widgets_banner_albums import AlbumsBannerWidget
-        RoundedArtistsView.__init__(self, ViewType.SCROLLED | ViewType.OVERLAY)
-        self.__banner = AlbumsBannerWidget([Type.ARTISTS], [], self._view_type)
+        from lollypop.widgets_banner_flowbox import FlowboxBannerWidget
+        RoundedArtistsView.__init__(self, storage_type,
+                                    view_type | ViewType.OVERLAY)
+        self.__banner = FlowboxBannerWidget([Type.ARTISTS], [], self.view_type)
         self.__banner.show()
         self.__banner.connect("play-all", self.__on_banner_play_all)
         self.add_widget(self._box, self.__banner)
 
-    @property
-    def args(self):
+#######################
+# PROTECTED           #
+#######################
+    def _on_map(self, widget):
         """
-            Get default args for __class__
-            @return {}
+            Set initial view state
+            @param widget as GtK.Widget
         """
-        return {}
+        RoundedArtistsView._on_map(self, widget)
+        if self.view_type & ViewType.SCROLLED:
+            self.scrolled.grab_focus()
 
 #######################
 # PRIVATE             #
@@ -217,7 +205,8 @@ class RoundedArtistsViewWithBanner(RoundedArtistsView):
             @param banner as AlbumsBannerWidget
             @param random as bool
         """
-        album_ids = App().albums.get_ids([], [], True, OrderBy.ARTIST)
+        album_ids = App().albums.get_ids([], [], self.storage_type,
+                                         False, OrderBy.ARTIST)
         if not album_ids:
             return
         albums = [Album(album_id) for album_id in album_ids]
