@@ -15,24 +15,23 @@ from gi.repository import Gtk
 from locale import strcoll
 
 from lollypop.view_lazyloading import LazyLoadingView
-from lollypop.helper_filtering import FilteringHelper
 from lollypop.helper_gestures import GesturesHelper
 from lollypop.define import ViewType, App
-from lollypop.utils import get_font_height, popup_widget
+from lollypop.utils import get_font_height, popup_widget, set_cursor_type
 
 
-class FlowBoxView(FilteringHelper, LazyLoadingView, GesturesHelper):
+class FlowBoxView(LazyLoadingView, GesturesHelper):
     """
         Lazy loading FlowBox
     """
 
-    def __init__(self, view_type=ViewType.SCROLLED):
+    def __init__(self, storage_type, view_type=ViewType.SCROLLED):
         """
             Init flowbox view
+            @param storage_type as StorageType
             @param view_type as ViewType
         """
-        LazyLoadingView.__init__(self, view_type)
-        FilteringHelper.__init__(self)
+        LazyLoadingView.__init__(self, storage_type, view_type)
         self._items = []
         self.__hovered_child = None
         self.__font_height = get_font_height()
@@ -57,22 +56,28 @@ class FlowBoxView(FilteringHelper, LazyLoadingView, GesturesHelper):
         """
         LazyLoadingView.populate(self, items)
 
-    def add_value(self, item_id):
+    def add_value(self, value):
         """
-            Insert item
-            @param item_id as int
+            Append value
+            @param value as object
         """
-        # Setup sort on insert
         self._box.set_sort_func(self._sort_func)
-        LazyLoadingView.populate(self, [item_id])
+        LazyLoadingView.populate(self, [value])
 
-    def remove_value(self, item_id):
+    def add_value_unsorted(self, value):
+        """
+            Add value unsorted
+            @param value as object
+        """
+        LazyLoadingView.populate(self, [value])
+
+    def remove_value(self, value):
         """
             Remove value
-            @param item_id as int
+            @param value as object
         """
         for child in self._box.get_children():
-            if child.data == item_id:
+            if child.data == value:
                 child.destroy()
                 break
 
@@ -83,17 +88,6 @@ class FlowBoxView(FilteringHelper, LazyLoadingView, GesturesHelper):
         for child in self._box.get_children():
             child.destroy()
 
-    def activate_child(self):
-        """
-            Activated typeahead row
-        """
-        self._box.unselect_all()
-        for row in self._box.get_children():
-            style_context = row.get_style_context()
-            if style_context.has_class("typeahead"):
-                row.activate()
-            style_context.remove_class("typeahead")
-
     @property
     def font_height(self):
         """
@@ -136,14 +130,6 @@ class FlowBoxView(FilteringHelper, LazyLoadingView, GesturesHelper):
         """
         return strcoll(widget1.sortname, widget2.sortname)
 
-    def _on_current_changed(self, player):
-        """
-            Update children state
-            @param player as Player
-        """
-        for child in self._box.get_children():
-            child.set_selection()
-
     def _on_child_activated(self, flowbox, child):
         pass
 
@@ -157,7 +143,7 @@ class FlowBoxView(FilteringHelper, LazyLoadingView, GesturesHelper):
             self.pause()
             children = self._box.get_children()
             for child in children:
-                child.set_view_type(self._view_type)
+                child.set_view_type(self.view_type)
                 child.disable_artwork()
                 self.queue_lazy_loading(child)
             self.lazy_loading()
@@ -200,7 +186,6 @@ class FlowBoxView(FilteringHelper, LazyLoadingView, GesturesHelper):
 #######################
 # PRIVATE             #
 #######################
-
     def __popup_menu(self, child):
         """
             Popup album menu at position
@@ -209,7 +194,7 @@ class FlowBoxView(FilteringHelper, LazyLoadingView, GesturesHelper):
         menu_widget = self._get_menu_widget(child)
         if menu_widget is not None:
             menu_widget.show()
-            popup_widget(menu_widget, child.artwork)
+            popup_widget(menu_widget, child.artwork, None, None, None)
 
     def __unselect_selected(self):
         """
@@ -219,6 +204,7 @@ class FlowBoxView(FilteringHelper, LazyLoadingView, GesturesHelper):
                 self.__hovered_child.artwork is not None:
             self.__hovered_child.artwork.unset_state_flags(
                 Gtk.StateFlags.VISITED)
+            set_cursor_type(self.__hovered_child, "left_ptr")
             self.__hovered_child = None
 
     def __on_box_motion(self, event_controller, x, y):
@@ -235,5 +221,6 @@ class FlowBoxView(FilteringHelper, LazyLoadingView, GesturesHelper):
             child.artwork.set_state_flags(Gtk.StateFlags.VISITED, False)
             self.__unselect_selected()
             self.__hovered_child = child
+            set_cursor_type(child)
         else:
             self.__unselect_selected()
