@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -12,6 +12,7 @@
 
 
 from lollypop.define import App, Type
+from lollypop.utils import get_default_storage_type
 from lollypop.widgets_albums_rounded import RoundedAlbumsWidget
 
 
@@ -67,15 +68,20 @@ class AlbumsDecadeWidget(RoundedAlbumsWidget):
             Get album ids
             @return [int]
         """
+        storage_type = get_default_storage_type()
         album_ids = []
         for year in self._data:
-            album_ids += App().albums.get_albums_for_year(year,
-                                                          self._ALBUMS_COUNT)
+            album_ids += App().albums.get_ids_for_year(year,
+                                                       storage_type,
+                                                       self._ALBUMS_COUNT,
+                                                       True)
             l = len(album_ids)
             if l < self._ALBUMS_COUNT:
-                album_ids += App().albums.get_compilations_for_year(
+                album_ids += App().albums.get_compilation_ids_for_year(
                                                        year,
-                                                       self._ALBUMS_COUNT)
+                                                       storage_type,
+                                                       self._ALBUMS_COUNT,
+                                                       True)
         return album_ids
 
 #######################
