@@ -12,7 +12,7 @@
 
 from gi.repository import Gtk
 
-from lollypop.define import App, ArtSize, MARGIN, Type, ViewType, Size
+from lollypop.define import App, ArtSize, Type, ViewType, Size
 from lollypop.define import ArtBehaviour
 from lollypop.widgets_rating import RatingWidget
 from lollypop.widgets_loved import LovedWidget
@@ -30,15 +30,16 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
     """
 
     @signals_map
-    def __init__(self, album, view_type=ViewType.DEFAULT):
+    def __init__(self, album, storage_type, view_type):
         """
             Init cover widget
             @param album
+            @param storage_type as int
             @param view_type as ViewType
         """
         BannerWidget.__init__(self, view_type)
-        self.__cloud_image = None
         self.__album = album
+        self.__storage_type = storage_type
         builder = Gtk.Builder()
         builder.add_from_resource("/org/gnome/Lollypop/AlbumBannerWidget.ui")
         builder.connect_signals(self)
@@ -51,6 +52,7 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         self.__play_button = builder.get_object("play_button")
         self.__add_button = builder.get_object("add_button")
         self.__menu_button = builder.get_object("menu_button")
+        self.__widget = builder.get_object("widget")
         if view_type & ViewType.OVERLAY:
             style = "banner-button"
         else:
@@ -64,12 +66,15 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         if view_type & ViewType.ALBUM:
             self.__artist_label.show()
             self.__artist_label.set_label(", ".join(album.artists))
+        else:
+            self.__title_label.set_opacity(0.8)
+            self.__year_label.set_opacity(0.7)
+            self.__duration_label.set_opacity(0.6)
+            self.__widget.get_style_context().add_class("album-banner")
         if album.year is not None:
             self.__year_label.set_label(str(album.year))
             self.__year_label.show()
-        duration = App().albums.get_duration(self.__album.id,
-                                             self.__album.genre_ids)
-        human_duration = get_human_duration(duration)
+        human_duration = get_human_duration(album.duration)
         self.__duration_label.set_text(human_duration)
         artist_eventbox = builder.get_object("artist_eventbox")
         artist_eventbox.connect("realize", set_cursor_type)
@@ -79,7 +84,6 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         year_eventbox.connect("realize", set_cursor_type)
         self.__gesture2 = GesturesHelper(
             year_eventbox, primary_press_callback=self._on_year_press)
-        self.__widget = builder.get_object("widget")
         self.__widget.attach(self.__cover_widget, 0, 0, 1, 3)
         self.__bottom_box = builder.get_object("bottom_box")
         self.__loved_widget = LovedWidget(album, Gtk.IconSize.INVALID)
@@ -88,7 +92,6 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         self.__rating_widget = RatingWidget(album, Gtk.IconSize.INVALID)
         self.__rating_widget.show()
         self.__bottom_box.pack_start(self.__rating_widget, 0, True, True)
-        self.__cover_widget.set_margin_start(MARGIN)
         if view_type & ViewType.OVERLAY:
             self._overlay.add_overlay(self.__widget)
             self._overlay.set_overlay_pass_through(self.__widget, True)
@@ -98,7 +101,9 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         return [
                 (App().art, "album-artwork-changed",
                  "_on_album_artwork_changed"),
-                (App().player, "playback-changed", "_on_playback_changed")
+                (App().player, "playback-added", "_on_playback_changed"),
+                (App().player, "playback-updated", "_on_playback_changed"),
+                (App().player, "playback-removed", "_on_playback_changed")
         ]
 
     def update_for_width(self, width):
@@ -139,18 +144,20 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         """
         from lollypop.widgets_menu import MenuBuilder
         from lollypop.menu_objects import AlbumMenu
-        menu = AlbumMenu(self.__album, self._view_type | ViewType.BANNER,
+        menu = AlbumMenu(self.__album,
+                         self.__storage_type,
+                         self.view_type,
                          App().window.is_adaptive)
         menu_widget = MenuBuilder(menu)
         menu_widget.show()
-        popup_widget(menu_widget, button)
+        popup_widget(menu_widget, button, None, None, button)
 
     def _on_play_button_clicked(self, button):
         """
             Play album
            @param button as Gtk.Button
         """
-        App().player.play_album(self.__album.get_with_skipping_allowed())
+        App().player.play_album(self.__album.clone(False))
 
     def _on_add_button_clicked(self, button):
         """
@@ -160,7 +167,7 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         if self.__album.id in App().player.album_ids:
             App().player.remove_album_by_id(self.__album.id)
         else:
-            App().player.add_album(self.__album.get_with_skipping_allowed())
+            App().player.add_album(self.__album.clone(False))
 
     def _on_album_artwork_changed(self, art, album_id):
         """
@@ -171,10 +178,9 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         if album_id == self.__album.id:
             self.__set_artwork()
 
-    def _on_playback_changed(self, player):
+    def _on_playback_changed(self, *ignore):
         """
             Update add button
-            @param player as Player
         """
         self.__update_add_button()
 
@@ -205,7 +211,7 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
             Set artwork on banner
         """
         if self._artwork is not None and\
-                self._view_type & ViewType.ALBUM and\
+                self.view_type & ViewType.ALBUM and\
                 App().animations:
             App().art_helper.set_album_artwork(
                             self.__album,
@@ -223,10 +229,10 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         """
         if self.__album.id in App().player.album_ids:
             self.__add_button.get_image().set_from_icon_name(
-                "list-remove-symbolic", Gtk.IconSize.LARGE_TOOLBAR)
+                "list-remove-symbolic", Gtk.IconSize.BUTTON)
         else:
             self.__add_button.get_image().set_from_icon_name(
-                "list-add-symbolic", Gtk.IconSize.LARGE_TOOLBAR)
+                "list-add-symbolic", Gtk.IconSize.BUTTON)
 
     def __set_internal_size(self):
         """
@@ -242,10 +248,14 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
                 context.remove_class(c)
 
         if self.width <= Size.SMALL:
-            art_size = None
+            # For album, we show artist name and we need place
+            if self.view_type & ViewType.ALBUM:
+                art_size = None
+            else:
+                art_size = ArtSize.MEDIUM
             icon_size = Gtk.IconSize.BUTTON
             cls_title = "text-medium"
-            cls_others = "text-small"
+            cls_others = "text-medium"
         elif self.width <= Size.MEDIUM:
             art_size = ArtSize.MEDIUM
             icon_size = Gtk.IconSize.BUTTON
