@@ -17,7 +17,6 @@ from gettext import gettext as _
 from lollypop.define import App, MARGIN, ViewType
 from lollypop.define import ArtBehaviour, Size
 from lollypop.widgets_banner import BannerWidget
-from lollypop.objects_radio import Radio
 from lollypop.utils import emit_signal
 from lollypop.helper_signals import SignalsHelper, signals_map
 
@@ -45,7 +44,7 @@ class LyricsBannerWidget(BannerWidget, SignalsHelper):
         self.__title_label.set_vexpand(True)
         self.__translate_button = Gtk.ToggleButton.new()
         image = Gtk.Image.new_from_icon_name(
-            "accessories-dictionary-symbolic", Gtk.IconSize.LARGE_TOOLBAR)
+            "accessories-dictionary-symbolic", Gtk.IconSize.BUTTON)
         image.show()
         self.__translate_button.set_image(image)
         self.__translate_button.show()
@@ -62,7 +61,7 @@ class LyricsBannerWidget(BannerWidget, SignalsHelper):
         grid.set_column_spacing(MARGIN)
         grid.add(self.__title_label)
         grid.add(self.__translate_button)
-        grid.set_margin_start(MARGIN)
+        grid.set_property("margin", MARGIN)
         grid.set_margin_end(MARGIN)
         self._overlay.add_overlay(grid)
         self._overlay.set_overlay_pass_through(grid, True)
@@ -150,26 +149,15 @@ class LyricsBannerWidget(BannerWidget, SignalsHelper):
         else:
             self._artwork.get_style_context().remove_class(
                 "default-banner")
-            if isinstance(App().player.current_track, Radio):
-                App().art_helper.set_radio_artwork(
-                        App().player.current_track.name,
-                        # +100 to prevent resize lag
-                        self.width + 100,
-                        self.height,
-                        self._artwork.get_scale_factor(),
-                        ArtBehaviour.BLUR_HARD |
-                        ArtBehaviour.DARKER,
-                        self._on_artwork)
-            else:
-                App().art_helper.set_album_artwork(
-                        App().player.current_track.album,
-                        # +100 to prevent resize lag
-                        self.width + 100,
-                        self.height,
-                        self._artwork.get_scale_factor(),
-                        ArtBehaviour.BLUR_HARD |
-                        ArtBehaviour.DARKER,
-                        self._on_artwork)
+            App().art_helper.set_album_artwork(
+                    App().player.current_track.album,
+                    # +100 to prevent resize lag
+                    self.width + 100,
+                    self.height,
+                    self._artwork.get_scale_factor(),
+                    ArtBehaviour.BLUR_HARD |
+                    ArtBehaviour.DARKER,
+                    self._on_artwork)
 
     def __set_internal_size(self):
         """
