@@ -10,7 +10,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, GObject, Gdk, GdkPixbuf, GLib
+from gi.repository import Gtk, GObject, Gdk, GdkPixbuf, GLib, Pango
 
 from lollypop.objects_album import Album
 from lollypop.define import App, ArtSize, ArtBehaviour, MARGIN
@@ -75,18 +75,27 @@ class MenuBuilder(Gtk.Stack, SignalsHelper):
         """
             Add pending widget to menu
         """
-        main = self.get_child_by_name("main")
-        if isinstance(main, Gtk.ScrolledWindow):
-            # scrolled -> viewport -> box
-            main = main.get_child().get_child()
         while self.__widgets_queue:
             widget = self.__widgets_queue.pop(0)
-            main.add(widget)
+            if widget.submenu_name is not None:
+                self.__add_menu_container(widget.submenu_name, True, True)
+                self.__boxes[widget.submenu_name].add(widget)
+                button = Gtk.ModelButton.new()
+                button.set_label(widget.submenu_name)
+                button.get_child().set_halign(Gtk.Align.START)
+                button.set_property("menu-name", widget.submenu_name)
+                button.show()
+                self.__boxes["main"].add(button)
+            else:
+                main = self.get_child_by_name("main")
+                if isinstance(main, Gtk.ScrolledWindow):
+                    # scrolled -> viewport -> box
+                    main = main.get_child().get_child()
+                main.add(widget)
 
-    def __add_menu(self, menu, menu_name, submenu, scrolled):
+    def __add_menu_container(self, menu_name, submenu, scrolled):
         """
-            Build menu
-            @param menu as Gio.Menu
+            Add menu container
             @param menu_name as str
             @param submenu as bool
             @param scrolled as bool
@@ -113,8 +122,19 @@ class MenuBuilder(Gtk.Stack, SignalsHelper):
                 button.set_property("menu-name", "main")
                 button.set_property("inverted", True)
                 button.set_label(menu_name)
+                button.get_child().set_halign(Gtk.Align.START)
                 button.show()
                 box.add(button)
+
+    def __add_menu(self, menu, menu_name, submenu, scrolled):
+        """
+            Create container and add menu
+            @param menu as Gio.Menu
+            @param menu_name as str
+            @param submenu as bool
+            @param scrolled as bool
+        """
+        self.__add_menu_container(menu_name, submenu, scrolled)
         menu_range = list(range(0, menu.get_n_items()))
         if submenu:
             self.__submenu_queue.append((menu, menu_name, menu_range))
@@ -195,7 +215,7 @@ class MenuBuilder(Gtk.Stack, SignalsHelper):
         button = Gtk.ModelButton.new()
         button.set_action_name(action.get_string())
         button.set_label(text.get_string())
-        button.set_alignment(0, 0.5)
+        button.get_child().set_halign(Gtk.Align.START)
         if close:
             button.connect("clicked",
                            lambda x: emit_signal(self, "hidden", True))
@@ -244,7 +264,7 @@ class MenuBuilder(Gtk.Stack, SignalsHelper):
         button = Gtk.ModelButton.new()
         button.set_property("menu-name", submenu_name)
         button.set_label(text.get_string())
-        button.set_alignment(0, 0.5)
+        button.get_child().set_halign(Gtk.Align.START)
         button.show()
         self.__boxes[menu_name].add(button)
 
@@ -256,20 +276,24 @@ class MenuBuilder(Gtk.Stack, SignalsHelper):
             @param menu_name as str
         """
         button = Gtk.ModelButton.new()
-        button.set_alignment(0, 0.5)
         button.connect("clicked", lambda x: emit_signal(self, "hidden", True))
         button.show()
         label = Gtk.Label.new()
         label.set_markup(text)
+        label.set_ellipsize(Pango.EllipsizeMode.END)
         label.show()
         artwork = Gtk.Image.new_from_icon_name(icon_name,
                                                Gtk.IconSize.INVALID)
         artwork.set_pixel_size(ArtSize.SMALL)
         artwork.show()
+        close_image = Gtk.Image.new_from_icon_name("pan-up-symbolic",
+                                                   Gtk.IconSize.BUTTON)
+        close_image.show()
         grid = Gtk.Grid()
         grid.set_column_spacing(MARGIN)
         grid.add(artwork)
         grid.add(label)
+        grid.add(close_image)
         button.set_image(grid)
         button.get_style_context().add_class("padding")
         self.__boxes[menu_name].add(button)
@@ -282,17 +306,22 @@ class MenuBuilder(Gtk.Stack, SignalsHelper):
             @param menu_name as str
         """
         button = Gtk.ModelButton.new()
-        button.set_alignment(0, 0.5)
         button.connect("clicked", lambda x: emit_signal(self, "hidden", True))
         button.show()
         label = Gtk.Label.new()
         label.set_markup(text)
+        label.set_ellipsize(Pango.EllipsizeMode.END)
         label.show()
         artwork = Gtk.Image.new()
+        close_image = Gtk.Image.new_from_icon_name("pan-up-symbolic",
+                                                   Gtk.IconSize.BUTTON)
+        close_image.show()
         grid = Gtk.Grid()
+        grid.set_halign(Gtk.Align.START)
         grid.set_column_spacing(MARGIN)
         grid.add(artwork)
         grid.add(label)
+        grid.add(close_image)
         button.set_image(grid)
         button.get_style_context().add_class("padding")
         App().art_helper.set_album_artwork(
@@ -313,17 +342,21 @@ class MenuBuilder(Gtk.Stack, SignalsHelper):
             @param menu_name as str
         """
         button = Gtk.ModelButton.new()
-        button.set_alignment(0, 0.5)
         button.connect("clicked", lambda x: emit_signal(self, "hidden", True))
         button.show()
         label = Gtk.Label.new()
         label.set_markup(text)
+        label.set_ellipsize(Pango.EllipsizeMode.END)
         label.show()
         artwork = Gtk.Image.new()
+        close_image = Gtk.Image.new_from_icon_name("pan-up-symbolic",
+                                                   Gtk.IconSize.BUTTON)
+        close_image.show()
         grid = Gtk.Grid()
         grid.set_column_spacing(MARGIN)
         grid.add(artwork)
         grid.add(label)
+        grid.add(close_image)
         button.set_image(grid)
         button.get_style_context().add_class("padding")
         artist_name = App().artists.get_name(artist_id)
@@ -360,18 +393,22 @@ class MenuBuilder(Gtk.Stack, SignalsHelper):
                 del rounded
                 artwork.show()
         button = Gtk.ModelButton.new()
-        button.set_alignment(0, 0.5)
         button.connect("clicked", lambda x: emit_signal(self, "hidden", True))
         button.show()
         label = Gtk.Label.new()
+        label.set_ellipsize(Pango.EllipsizeMode.END)
         label.set_markup(text)
         label.show()
         artwork = Gtk.Image.new()
         artwork.get_style_context().add_class("light-background")
+        close_image = Gtk.Image.new_from_icon_name("pan-up-symbolic",
+                                                   Gtk.IconSize.BUTTON)
+        close_image.show()
         grid = Gtk.Grid()
         grid.set_column_spacing(MARGIN)
         grid.add(artwork)
         grid.add(label)
+        grid.add(close_image)
         button.set_image(grid)
         button.get_style_context().add_class("padding")
         App().task_helper.run(
@@ -405,5 +442,5 @@ class MenuBuilder(Gtk.Stack, SignalsHelper):
             self.set_transition_type(Gtk.StackTransitionType.SLIDE_LEFT)
             self.set_size_request(-1, -1)
         else:
-            self.set_size_request(-1, 300)
+            self.set_size_request(300, 400)
             self.set_transition_type(Gtk.StackTransitionType.SLIDE_RIGHT)
