@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,7 +16,7 @@ from gettext import gettext as _
 from time import time
 
 from lollypop.objects_track import Track
-from lollypop.define import App, Type
+from lollypop.define import App
 from lollypop.logger import Logger
 
 
@@ -91,16 +91,14 @@ class RatingWidget(Gtk.Bin):
             @param event as Gdk.Event (can be None)
         """
         user_rating = True
-        rate = self.__object.get_rate()
-        # -1 for compatiblity with previous release
-        if rate in [0, -1]:
+        rate = self.__object.rate
+        if rate < 1:
             rate = self.__object.get_popularity()
             user_rating = False
-        if rate < 1:
             for i in range(5):
                 self._stars[i].set_opacity(0.2)
                 self._stars[i].get_style_context().remove_class("selected")
-        else:
+        if rate > 0:
             star = self.__star_from_rate(rate)
             # Select wanted star
             for idx in range(0, star):
@@ -122,9 +120,8 @@ class RatingWidget(Gtk.Bin):
             @param event as Gdk.Event
         """
         user_rating = True
-        rate = self.__object.get_rate()
-        # -1 for compatiblity with previous release
-        if rate in [0, -1]:
+        rate = self.__object.rate
+        if rate < 1:
             rate = self.__object.get_popularity()
             user_rating = False
         max_star = self.__star_from_rate(rate)
@@ -136,9 +133,11 @@ class RatingWidget(Gtk.Bin):
         pop = position + 1
         if event.button != 1:
             self.__object.set_popularity(pop)
+            self.__object.set_rate(0)
+            self._on_leave_notify_event(None, None)
         elif pop == 0 or pop == max_star:
             if user_rating:
-                self.__object.set_rate(Type.NONE)
+                self.__object.set_rate(0)
             else:
                 self.__object.set_popularity(0)
             self._on_leave_notify_event(None, None)
