@@ -10,25 +10,28 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, GLib, Gio
+from gi.repository import Gtk, GLib, Gio, Gdk
 
 from gettext import gettext as _
 
-from lollypop.define import App, NetworkAccessACL, StorageType
-from lollypop.utils import get_network_available
-from lollypop.logger import Logger
+from lollypop.define import App, NetworkAccessACL
+from lollypop.define import LASTFM_API_KEY
+from lollypop.helper_passwords import PasswordsHelper
+from lollypop.helper_signals import SignalsHelper, signals_map
 
 
-class WebSettingsWidget(Gtk.Bin):
+class WebSettingsWidget(Gtk.Bin, SignalsHelper):
     """
         Widget allowing user to configure web providers
     """
 
+    @signals_map
     def __init__(self):
         """
             Init widget
         """
         Gtk.Bin.__init__(self)
+        self.__cancellable = Gio.Cancellable()
         builder = Gtk.Builder()
         builder.add_from_resource("/org/gnome/Lollypop/SettingsWeb.ui")
 
@@ -39,29 +42,12 @@ class WebSettingsWidget(Gtk.Bin):
                           (builder.get_object("google_view"),
                            builder.get_object("google_error_label"),
                            None,
+                           False),
+                          (builder.get_object("lastfm_view"),
+                           builder.get_object("lastfm_error_label"),
+                           NetworkAccessACL["LASTFM"],
                            False)]
 
-        # First check lastfm support is available
-        if App().lastfm is None:
-            builder.get_object("lastfm_error_label").set_text(
-                _("You need to install pylast and gi secret"))
-            builder.get_object("librefm_error_label").set_text(
-                _("You need to install pylast and gi secret"))
-            builder.get_object("lastfm_error_label").set_opacity(1)
-            builder.get_object("librefm_error_label").set_opacity(1)
-            builder.get_object("lastfm_view").set_sensitive(False)
-            builder.get_object("librefm_view").set_sensitive(False)
-
-        else:
-            self.__widgets += [(builder.get_object("lastfm_view"),
-                                builder.get_object("lastfm_error_label"),
-                                NetworkAccessACL["LASTFM"],
-                                App().lastfm.is_goa),
-                               (builder.get_object("librefm_view"),
-                                builder.get_object("librefm_error_label"),
-                                NetworkAccessACL["LASTFM"],
-                                False)]
-
         # Web services access
         self.__acl_grid = builder.get_object("acl_grid")
         switch_network_access = builder.get_object("switch_network_access")
@@ -73,15 +59,13 @@ class WebSettingsWidget(Gtk.Bin):
             if acl & NetworkAccessACL[key]:
                 builder.get_object(key).set_state(True)
 
-        # Check web services access
-        self.__check_acls()
-
         #
         # Google tab
         #
         key = App().settings.get_value("cs-api-key").get_string() or\
             App().settings.get_default_value("cs-api-key").get_string()
-        builder.get_object("cs-entry").set_text(key)
+        self.__cs_entry = builder.get_object("cs-entry")
+        self.__cs_entry.set_text(key)
         uri = App().settings.get_value("invidious-server").get_string()
         recent_youtube_dl = App().settings.get_value("recent-youtube-dl")
         self.__switch_youtube = builder.get_object("switch_youtube")
@@ -98,31 +82,26 @@ class WebSettingsWidget(Gtk.Bin):
             "listenbrainz-user-token").get_string()
         builder.get_object("listenbrainz_user_token_entry").set_text(token)
 
-        from lollypop.helper_passwords import PasswordsHelper
-        helper = PasswordsHelper()
-
-        #
-        # Last.fm tab
-        #
-        self.__lastfm_test_image = builder.get_object("lastfm_test_image")
-        self.__lastfm_login = builder.get_object("lastfm_login")
-        self.__lastfm_password = builder.get_object("lastfm_password")
-        helper.get("lastfm", self.__on_get_password)
-
-        #
-        # Libre.fm tab
-        #
-        self.__librefm_test_image = builder.get_object("librefm_test_image")
-        self.__librefm_login = builder.get_object("librefm_login")
-        self.__librefm_password = builder.get_object("librefm_password")
-        helper.get("librefm", self.__on_get_password)
-
         self.add(builder.get_object("widget"))
+
+        # Check web services access
+        self.__check_acls()
+
         builder.connect_signals(self)
-        App().settings.connect("changed::network-access",
-                               lambda x, y: self.__check_acls())
-        App().settings.connect("changed::network-access-acl",
-                               lambda x, y: self.__check_acls())
+        self.connect("unmap", self.__on_unmap)
+
+        self.__passwords_helper = PasswordsHelper()
+        self.__passwords_helper.get("LASTFM", self.__on_get_password,
+                                    builder.get_object("lastfm_button"))
+        self.__passwords_helper.get("LIBREFM", self.__on_get_password,
+                                    builder.get_object("librefm_button"))
+
+        return [
+            (App().settings, "changed::network-access",
+             "_on_network_access_changed"),
+            (App().settings, "changed::network-access-acl",
+             "_on_network_access_changed"),
+        ]
 
 #######################
 # PROTECTED           #
@@ -143,31 +122,32 @@ class WebSettingsWidget(Gtk.Bin):
         value = entry.get_text().strip()
         App().settings.set_value("listenbrainz-user-token",
                                  GLib.Variant("s", value))
-        App().load_listenbrainz()
 
-    def _on_lastfm_test_btn_clicked(self, button):
+    def _on_lastfm_button_clicked(self, button):
         """
-            Test lastfm connection
+            Connect to lastfm
             @param button as Gtk.Button
         """
-        self.__update_fm_settings("lastfm")
-        if not get_network_available():
-            self.__lastfm_test_image.set_from_icon_name(
-                "computer-fail-symbolic",
-                Gtk.IconSize.MENU)
-            return
+        if button.get_tooltip_text():
+            button.set_tooltip_text("")
+            button.set_label(_("Connect"))
+            App().ws_director.token_ws.clear_token("LASTFM", True)
+        else:
+            button.set_sensitive(False)
+            App().task_helper.run(self.__get_lastfm_token, button, "LASTFM")
 
-    def _on_librefm_test_btn_clicked(self, button):
+    def _on_librefm_button_clicked(self, button):
         """
             Test librefm connection
             @param button as Gtk.Button
         """
-        self.__update_fm_settings("librefm")
-        if not get_network_available():
-            self.__librefm_test_image.set_from_icon_name(
-                "computer-fail-symbolic",
-                Gtk.IconSize.MENU)
-            return
+        if button.get_tooltip_text():
+            button.set_tooltip_text("")
+            button.set_label(_("Connect"))
+            App().ws_director.token_ws.clear_token("LIBREFM", True)
+        else:
+            button.set_sensitive(False)
+            App().task_helper.run(self.__get_lastfm_token, button, "LIBREFM")
 
     def _on_switch_youtube_state_set(self, widget, state):
         """
@@ -178,7 +158,7 @@ class WebSettingsWidget(Gtk.Bin):
         App().settings.set_value("recent-youtube-dl",
                                  GLib.Variant("b", state))
         if Gio.NetworkMonitor.get_default().get_network_available() and state:
-            from lollypop.utils import install_youtube_dl
+            from lollypop.utils_file import install_youtube_dl
             App().task_helper.run(install_youtube_dl)
 
     def _on_entry_invidious_changed(self, entry):
@@ -214,17 +194,44 @@ class WebSettingsWidget(Gtk.Bin):
             acl &= ~NetworkAccessACL[key]
         acl = App().settings.set_value("network-access-acl",
                                        GLib.Variant("i", acl))
-        if key == "SPOTIFY" and not state:
-            for storage_type in [StorageType.SPOTIFY_NEW_RELEASES,
-                                 StorageType.SPOTIFY_SIMILARS]:
-                App().tracks.del_old_for_storage_type(storage_type, 0)
-            App().tracks.clean()
-            App().albums.clean()
-            App().artists.clean()
+
+    def _on_network_access_changed(self, *ignore):
+        self.__check_acls()
 
 #######################
 # PRIVATE             #
 #######################
+    def __get_lastfm_token(self, button, service):
+        """
+            Get Last.fm token
+            @param button as Gtk.Button
+            @param service as str
+            @thread safe
+        """
+        def on_token(token, service):
+            self.__passwords_helper.clear(service,
+                                          self.__passwords_helper.store,
+                                          service,
+                                          service,
+                                          token)
+            validation_token = token.replace("validation:", "")
+            if service == "LIBREFM":
+                uri = "http://libre.fm/api/auth?api_key=%s&token=%s" % (
+                    LASTFM_API_KEY, validation_token)
+            else:
+                uri = "http://www.last.fm/api/auth?api_key=%s&token=%s" % (
+                    LASTFM_API_KEY, validation_token)
+            GLib.idle_add(show_uri, uri)
+            # Force web service to validate token
+            App().ws_director.token_ws.clear_token(service)
+
+        def show_uri(uri):
+            Gtk.show_uri_on_window(App().window, uri, Gdk.CURRENT_TIME)
+
+        App().ws_director.token_ws.clear_token(service, True)
+        App().ws_director.token_ws.get_lastfm_auth_token(
+            service, self.__cancellable, on_token)
+
     def __check_acls(self):
         """
             Check network ACLs
@@ -243,94 +250,25 @@ class WebSettingsWidget(Gtk.Bin):
             else:
                 view.set_sensitive(True)
                 label.set_opacity(0)
+        self.__switch_youtube.set_sensitive(acls & NetworkAccessACL["YOUTUBE"])
+        self.__cs_entry.set_sensitive(acls & NetworkAccessACL["YOUTUBE"] or
+                                      acls & NetworkAccessACL["GOOGLE"])
 
-    def __update_fm_settings(self, name):
+    def __on_unmap(self, widget):
         """
-            Update *fm settings
-            @param name as str (librefm/lastfm)
+            Cancel current tasks and clear token
+            @param widget as Gtk.Widget
         """
-        fm = None
-        for scrobbler in App().scrobblers:
-            if scrobbler.service_name == name:
-                fm = scrobbler
-                break
-        if fm is None:
-            return
-        elif name == "librefm":
-            callback = self.__test_librefm_connection
-            login = self.__librefm_login.get_text()
-            password = self.__librefm_password.get_text()
-        else:
-            callback = self.__test_lastfm_connection
-            login = self.__lastfm_login.get_text()
-            password = self.__lastfm_password.get_text()
-        try:
-            if fm is not None and login and password:
-                from lollypop.helper_passwords import PasswordsHelper
-                helper = PasswordsHelper()
-                helper.clear(name,
-                             helper.store,
-                             name,
-                             login,
-                             password,
-                             self.__on_password_store,
-                             fm,
-                             callback)
-        except Exception as e:
-            Logger.error("SettingsDialog::__update_fm_settings(): %s" % e)
+        self.__cancellable.cancel()
 
-    def __test_lastfm_connection(self, result, fm):
+    def __on_get_password(self, attributes, password, service, button):
         """
-            Test lastfm connection
-            @param result as None
-            @param fm as LastFM
+            Set button state
+            @param attributes as {}
+            @param password as str
+            @param service as str
+            @param button as Gtk.Button
         """
-        if fm.available:
-            self.__lastfm_test_image.set_from_icon_name(
-                "object-select-symbolic",
-                Gtk.IconSize.MENU)
-        else:
-            self.__lastfm_test_image.set_from_icon_name(
-                "computer-fail-symbolic",
-                Gtk.IconSize.MENU)
-
-    def __test_librefm_connection(self, result, fm):
-        """
-            Test librefm connection
-            @param result as None
-            @param fm as LastFM
-        """
-        if fm.available:
-            self.__librefm_test_image.set_from_icon_name(
-                "object-select-symbolic",
-                Gtk.IconSize.MENU)
-        else:
-            self.__librefm_test_image.set_from_icon_name(
-                "computer-fail-symbolic",
-                Gtk.IconSize.MENU)
-
-    def __on_password_store(self, source, result, fm, callback):
-        """
-            Connect service
-            @param source as GObject.Object
-            @param result as Gio.AsyncResult
-            @param fm as LastFM
-            @param callback as function
-        """
-        fm.connect_service(True, callback, fm)
-
-    def __on_get_password(self, attributes, password, name):
-        """
-             Set password label
-             @param attributes as {}
-             @param password as str
-             @param name as str
-        """
-        if attributes is None:
-            return
-        if name == "librefm":
-            self.__librefm_login.set_text(attributes["login"])
-            self.__librefm_password.set_text(password)
-        else:
-            self.__lastfm_login.set_text(attributes["login"])
-            self.__lastfm_password.set_text(password)
+        if attributes is not None:
+            button.set_label(_("Disconnect"))
+            button.set_tooltip_text(attributes["login"])
